<?php

// GeniuXs AI v1.0 for LFM Scripts
// Copyright 2023 Logiscape LLC. All Rights Reserved.
// https://geniuxs.com

// Any modification, distribution, or use of this file is governed
// by the current terms of service at geniuxs.com and any additional
// license agreements for the script this file was included with.

require_once "../inc/filter.php";


// Prevent anyone who isn't logged in from opening this page
include "../inc/checkauth.php";
if(!isset($_SESSION["adminid"])) { exit; };

if (!isset($_GET['aiid']) || !is_numeric($_GET['aiid']) || $_GET['aiid'] < 1) {
	echo("Invalid AI ID");
	exit;
} else {
	$aiid = $_GET['aiid'];
}

function estimate_tokens($text) {
	$word_count = str_word_count($text);
	$char_count = strlen($text);
	$tokens_count_word_est = $word_count / 0.75;
	$tokens_count_char_est = $char_count / 4.0;
	// Include additional tokens for spaces and punctuation marks
	$additional_tokens = preg_match_all("/[\s.,!?;]/", $text);
	$tokens_count_word_est += $additional_tokens;
	$tokens_count_char_est += $additional_tokens;
	$output = max($tokens_count_word_est, $tokens_count_char_est);
	return intval($output);
}

function estimate_db_tokens($aidata) {
	$testtext = $aidata['descr']."\n\n";
	foreach ($aidata['tables'] as $tableName=>$value) {
        	$testtext .= "Table: ".$tableName."\nPurpose: ".$aidata['tables'][$tableName]['descr']."\n\nColumns:\n\n";
        	$columns = $aidata['tables'][$tableName]['columns'];
                foreach ($columns as $columnName=>$value) {
                	$testtext .= $columnName." (".$aidata['tables'][$tableName]['columns'][$columnName]['type']."): ".$aidata['tables'][$tableName]['columns'][$columnName]['descr']."\n";
                }
                if ($tableName == 'oto_membertypes') {
                	$testtext .= "\nCurrent membership levels:\n";
                	$getaccounts=lfmsql_query("Select mtid, accname from `".$prefix."membertypes` order by mtid asc");
			while($row=lfmsql_fetch_array($getaccounts)) {
				$testtext .= 'mtid #'.$row['mtid'].' = "'.$row['accname'].'"'."\n";
			}
                }
                $testtext .= "\n";
        }
        if (count($aidata['relationships']) > 0) {
        	$testtext .= "\nRelationships:\n";
        	foreach ($aidata['relationships'] as $relationshipindex=>$relationshipvalue) {
			$testtext .= $relationshipvalue['relationship1']." references ".$relationshipvalue['relationship2']."\n";
		}
        }
        
        return estimate_tokens($testtext);
}

function estimate_training_tokens($aidata) {
	$testtext = "";
	foreach ($aidata['training'] as $trainingindex=>$value) {
        	$testtext .= "Message: ".$aidata['training'][$trainingindex]['message']."\n";
        	$testtext .= "Response: ".$aidata['training'][$trainingindex]['response']."\n";
        }
        return estimate_tokens($testtext);
}
    
    	$getaidata = lfmsql_query("SELECT aidata FROM `geniuxs_ai` WHERE id='".$aiid."'") or die(lfmsql_error());
    	if (lfmsql_num_rows($getaidata) < 1) {
    		echo("AI ID not found");
    		exit;
    	}
    	$aidata = lfmsql_result($getaidata, 0, "aidata");
    	$aidata = json_decode($aidata, true);
    
    
	if (!isset($aidata) || !is_array($aidata)) {
		$aidata = array();
	}
	
	if (!isset($aidata['descr'])) {
		$aidata['descr'] = '';
	}
	
	if (!isset($aidata['tables']) || !is_array($aidata['tables'])) {
		$aidata['tables'] = array();
	}
	
	if (!isset($aidata['relationships']) || !is_array($aidata['relationships'])) {
		$aidata['relationships'] = array();
	}

	if (!isset($aidata['training']) || !is_array($aidata['training'])) {
		$aidata['training'] = array();
	}
	
	if (isset($_GET['action']) && $_GET['action'] == "checkmemory") {
		echo(estimate_db_tokens($aidata).'/'.estimate_training_tokens($aidata));
		exit;
	}
	
    if (isset($_GET['action']) && $_GET['action'] == "selectcolumns") {
    	
    	$aidata['tables'] = array();
    	
        $getdbtables = lfmsql_query("SHOW TABLES");
        while ($table = lfmsql_fetch_array($getdbtables)) {
            $tableName = $table[0];
            if (isset($_POST[$tableName])) {
              	
                $columns = $_POST[$tableName];
                
                $aidata['tables'][$tableName] = array();
                $aidata['tables'][$tableName]['descr'] = base64_decode($_POST[$tableName.'_tabledescr']);
                $aidata['tables'][$tableName]['columns'] = array();
                
                foreach ($columns as $column) {
                    list($columnName, $columnType, $columnDescr) = explode(":", $column);
                    $aidata['tables'][$tableName]['columns'][$columnName]['type'] = $columnType;
                    $aidata['tables'][$tableName]['columns'][$columnName]['descr'] = base64_decode($columnDescr);
                }
            }
        }
        $aidata = json_encode($aidata);
        lfmsql_query("UPDATE `geniuxs_ai` SET aidata='".addslashes($aidata)."' WHERE id='".$aiid."'") or die(lfmsql_error());
        
        echo("Success");
	exit;
    }
    
    if (isset($_GET['action']) && $_GET['action'] == "viewcolumnform") {
    	
    	$formcode = '
    	    	
    	<form name="columnform" method="POST">
        <h2>Select Tables and Columns:</h2>
        
        <p>Select the tables and columns in your database that you want the AI to be able to access. You should only select what is relevant to the types of requests you intend your users will ask the AI. You can click the button below to preload your AI with the tables related to basic member information.</p>
        
        <button style="margin-top:10px; width:100%; text-align:left;" type="button" class="btn btn-outline-primary btn-lg" onclick="quickSelects(1)">SELECT MEMBER TABLES<br><span style="color:black; font-size:14px;">This preloads the tables and descriptions for your AI to answer basic questions about member accounts</span></button><br><br>
        
        <ul class="geniuxs_create_list_mainmenu">';
        
        $getdbtables = lfmsql_query("SHOW TABLES");
        while ($table = lfmsql_fetch_array($getdbtables)) {
            $tableName = $table[0];
		
		if (isset($aidata['tables'][$tableName]['columns']) && is_array($aidata['tables'][$tableName]['columns'])) {
			$toplinkclass = ' active';
			$expanderclass = '';
			$tableboxchecked = ' checked';
			$columnlistclass = ' show';
		} else {
			$toplinkclass = ' collapsed';
			$expanderclass = ' collapsed';
			$tableboxchecked = '';
			$columnlistclass = '';
		}
		
		$formcode .= '<li class="geniuxs_create_list_toplink'.$toplinkclass.'">';
		
		$formcode .= "<input class='geniuxs_create_checkbox' type='checkbox' id='{$tableName}_checkbox' onclick='toggleTablebox(this, \"{$tableName}_column\")'".$tableboxchecked.">";
            	$formcode .= "<label for='{$tableName}_checkbox' class='table-name'>{$tableName}</label>";
		
		$formcode .= '<a href="#'.$tableName.'_cat" data-toggle="collapse" class="geniuxs_create_expander'.$expanderclass.'"> </a>';
		
		$formcode .= "<input class='geniuxs_create_tabledescr_hdn' type='hidden' id='{$tableName}_tabledescr_hdn' name='{$tableName}_tabledescr' value='".base64_encode($aidata['tables'][$tableName]['descr'])."'>";
            
            $formcode .= '<ul id="'.$tableName.'_cat" class="list-unstyled collapse'.$columnlistclass.'">';
		
		
            $columns = lfmsql_query("SHOW COLUMNS FROM {$tableName}");
            while ($column = lfmsql_fetch_assoc($columns)) {
                $columnName = $column["Field"];
                $columnType = $column["Type"];
                
                if (strpos($columnType, '(') !== false) {
                	$columnTypearr = explode('(', $columnType);
                	$columnType = trim($columnTypearr[0]);
                }
                
                $columnKey = $column["Key"] == "PRI" ? ", Primary Key" : "";
                
                $sublinkclass = '';
                $columnboxchecked = '';
                $columnDescr = '';
                if (isset($aidata['tables'][$tableName]['columns'][$columnName])) {
                	$sublinkclass = ' active';
                	$columnboxchecked = ' checked';
                	$columnDescr = base64_encode($aidata['tables'][$tableName]['columns'][$columnName]['descr']);
                }
                
                $formcode .= '<li class="geniuxs_create_list_sublink '.$sublinkclass.'">';
                
                $formcode .= "<input type='checkbox' id='{$tableName}_{$columnName}_checkbox' name='{$tableName}[]' value='{$columnName}:{$columnType}{$columnKey}:{$columnDescr}' class='geniuxs_create_checkbox geniuxs_column_checkbox {$tableName}_column' onclick='toggleColumnbox(this)'".$columnboxchecked.">";
                $formcode .= "<label class='column-name' for='{$tableName}_{$columnName}_checkbox'>{$columnName} ({$columnType}{$columnKey})</label>";
                $formcode .= "</li>";
            }

            $formcode .= "</ul>";
            
            $formcode .= '</li>';
        }
	
	$formcode .= '</ul>
	
        <button type="button" class="btn btn-primary" onclick="submitSelectColumns()">Save</button>
    </form>';
    
    	echo($formcode);
    	exit;
    }
    
    if (isset($_GET['action']) && $_GET['action'] == "savedescriptions") {
    	
        $tables = $aidata['tables'];
        foreach ($tables as $tableName=>$value) {
            if (isset($_POST[$tableName.'_tabledescr'])) {
              	
              	$aidata['tables'][$tableName]['descr'] = stripslashes($_POST[$tableName.'_tabledescr']);
              	
                $columns = $aidata['tables'][$tableName]['columns'];
                
                foreach ($columns as $columnName=>$value) {
                    if (isset($_POST[$tableName.'_'.$columnName.'_columndescr'])) {
    	                $aidata['tables'][$tableName]['columns'][$columnName]['descr'] = stripslashes($_POST[$tableName.'_'.$columnName.'_columndescr']);
    	            }
                }
            }
        }
        $aidata = json_encode($aidata);
        lfmsql_query("UPDATE `geniuxs_ai` SET aidata='".addslashes($aidata)."' WHERE id='".$aiid."'") or die(lfmsql_error());
        
        echo("Success");
	exit;
    }
    
    if (isset($_GET['action']) && $_GET['action'] == "loaddescriptions") {
    	
    	$formcode = '<form name="descriptionform" method="POST">
        <h2>Enter Tables and Columns Descriptions:</h2>
        
        <p>Now you should enter descriptions for the tables and columns you selected, to help the AI understand what they are. Some columns with obvious names, such as "email", may not require descriptions.</p>
        ';
        
        $tables = $aidata['tables'];
        
        foreach ($tables as $tableName=>$value) {
        	
        	$formcode .= "<div class=\"form-group row\">
        	<label class=\"col-sm-2 col-form-label text-right\" for='".$tableName."_tabledescr'><b>".$tableName."</b></label>
        	<div class=\"col-sm-10\">
        		<input class='form-control geniuxs_create_tabledescr' type='text' id='".$tableName."_tabledescr' name='".$tableName."_tabledescr' value=\"".htmlentities($aidata['tables'][$tableName]['descr'])."\">
        	</div>
        	</div>";
        	
        	$columns = $aidata['tables'][$tableName]['columns'];
                
                foreach ($columns as $columnName=>$value) {
                	
                	$formcode .= "<div class=\"form-group row\">
                	<label class=\"col-sm-2 col-form-label text-right\" for='".$tableName."_".$columnName."_columndescr'>".$columnName."</label>
                	<div class=\"col-sm-10\">
	                	<input class='form-control geniuxs_create_columndescr' type='text' id='".$tableName."_".$columnName."_columndescr' name='".$tableName."_".$columnName."_columndescr' value=\"".htmlentities($aidata['tables'][$tableName]['columns'][$columnName]['descr'])."\">
	                </div>
                	</div>";
                }
                
                $formcode .= "<hr>";
        }
	
	$formcode .= '</ul>
	
        <button type="button" class="btn btn-primary" onclick="submitDescriptions()">Save</button>
    </form>';
    
    	echo($formcode);
    	exit;
    }
    
    
        if (isset($_GET['action']) && $_GET['action'] == "addrelationship") {
    	
        $addrelationship = array();
        
        if (isset($_POST['addrelationship1']) && strlen($_POST['addrelationship1']) > 0 && isset($_POST['addrelationship2']) && strlen($_POST['addrelationship2']) > 0 && ($_POST['addrelationship1'] != $_POST['addrelationship2'])) {
        	
        	$addrelationship['relationship1'] = stripslashes($_POST['addrelationship1']);
        	$addrelationship['relationship2'] = stripslashes($_POST['addrelationship2']);
        	
        	$aidata['relationships'][] = $addrelationship;
        	
        	$aidata = json_encode($aidata);
        	lfmsql_query("UPDATE `geniuxs_ai` SET aidata='".addslashes($aidata)."' WHERE id='".$aiid."'") or die(lfmsql_error());
        	
        }
        
        echo("Success");
	exit;
    }
    
    if (isset($_GET['action']) && $_GET['action'] == "editrelationship" && isset($_GET['relationshipid']) && is_numeric($_GET['relationshipid'])) {
    	
    	$relationshipindex = $_GET['relationshipid'];
    	
        if (isset($_POST['editrelationship1_'.$relationshipindex]) && strlen($_POST['editrelationship1_'.$relationshipindex]) > 0 && isset($_POST['editrelationship2_'.$relationshipindex]) && strlen($_POST['editrelationship2_'.$relationshipindex]) > 0 && ($_POST['editrelationship1_'.$relationshipindex] != $_POST['editrelationship2_'.$relationshipindex]) && isset($aidata['relationships'][$relationshipindex])) {
       		
       		$aidata['relationships'][$relationshipindex]['relationship1'] = stripslashes($_POST['editrelationship1_'.$relationshipindex]);
        	$aidata['relationships'][$relationshipindex]['relationship2'] = stripslashes($_POST['editrelationship2_'.$relationshipindex]);
       		
       		$aidata = json_encode($aidata);
	        lfmsql_query("UPDATE `geniuxs_ai` SET aidata='".addslashes($aidata)."' WHERE id='".$aiid."'") or die(lfmsql_error());
        }
        
        echo("Success");
	exit;
    }
    
    if (isset($_GET['action']) && $_GET['action'] == "deleterelationship" && isset($_GET['relationshipid']) && is_numeric($_GET['relationshipid'])) {
    	
        if (isset($aidata['relationships'][$_GET['relationshipid']])) {
	        unset($aidata['relationships'][$_GET['relationshipid']]);
        	$aidata['relationships'] = array_values($aidata['relationships']);
        	$aidata = json_encode($aidata);
        	lfmsql_query("UPDATE `geniuxs_ai` SET aidata='".addslashes($aidata)."' WHERE id='".$aiid."'") or die(lfmsql_error());
        }
        
        echo("Success");
	exit;
    }
    
    if (isset($_GET['action']) && $_GET['action'] == "loadrelationships") {
    	
    	$formcode = '<form name="relationshipform" method="POST">
        <h2>Add relationship Examples:</h2>
        <p>Relationships help the AI find matching pieces of information between multiple tables. For example, the "mtype" column in the oto_members table matches with the "mtid" column in the oto_membertypes table. Therefore, you can add oto_members.mtype = oto_membertypes.mtid as a relationship.</p>
        ';
        
        foreach ($aidata['relationships'] as $relationshipindex=>$relationshipvalue) {
        	
        	$formcode .= '<div class="row">';
        	$formcode .= '<div class="col-5">';
        	$formcode .= "<select class='form-control geniuxs_create_editrelationship' id='editrelationship1_".$relationshipindex."' name='editrelationship1_".$relationshipindex."'>";
        		
        		$tables = $aidata['tables'];
		        foreach ($tables as $tableName=>$value) {
		        	$columns = $aidata['tables'][$tableName]['columns'];
		                foreach ($columns as $columnName=>$value) {
		                	$tablecolumn = $tableName.'.'.$columnName;
		                	if ($tablecolumn == $relationshipvalue['relationship1']) {
		                		$selected = ' selected';
		                	} else {
		                		$selected = '';
		                	}
        				$formcode .= "<option value=\"".$tablecolumn."\"".$selected.">".$tablecolumn."</option>";
        			}
        		}
        	$formcode .= "</select>";
        	$formcode .= '</div>';
        	
        	$formcode .= '<div class="col-5">';
        	$formcode .= "<select class='form-control geniuxs_create_editrelationship' id='editrelationship2_".$relationshipindex."' name='editrelationship2_".$relationshipindex."'>";
        		
        		$tables = $aidata['tables'];
		        foreach ($tables as $tableName=>$value) {
		        	$columns = $aidata['tables'][$tableName]['columns'];
		                foreach ($columns as $columnName=>$value) {
		                	$tablecolumn = $tableName.'.'.$columnName;
		                	if ($tablecolumn == $relationshipvalue['relationship2']) {
		                		$selected = ' selected';
		                	} else {
		                		$selected = '';
		                	}
        				$formcode .= "<option value=\"".$tablecolumn."\"".$selected.">".$tablecolumn."</option>";
        			}
        		}
        	$formcode .= "</select>";
        	$formcode .= '</div>';
        	
        	$formcode .= '<div class="col-2">';
        	$formcode .= '<button type="button" class="btn btn-primary mr-1" onclick="submitEditRelationship('.$relationshipindex.')">Save</button>';
        	$formcode .= '<button type="button" class="btn btn-danger" onclick="deleteRelationship('.$relationshipindex.')">Delete</button>';
        	$formcode .= '</div>';
        	$formcode .= '</div>';
        	
        	$formcode .= '<hr>';
        }
	
	$formcode .= '<div class="row">';
       	$formcode .= '<div class="col-5">';
	$formcode .= "<select class='form-control geniuxs_create_addrelationship' id='addrelationship1' name='addrelationship1'>";
        		
        		$tables = $aidata['tables'];
		        foreach ($tables as $tableName=>$value) {
		        	$columns = $aidata['tables'][$tableName]['columns'];
		                foreach ($columns as $columnName=>$value) {
		                	$tablecolumn = $tableName.'.'.$columnName;
        				$formcode .= "<option value=\"".$tablecolumn."\">".$tablecolumn."</option>";
        			}
        		}
        	$formcode .= "</select>";
        	$formcode .= '</div>';
        	
        	$formcode .= '<div class="col-5">';
        	$formcode .= "<select class='form-control geniuxs_create_addrelationship' id='addrelationship2' name='addrelationship2'>";
        		
        		$tables = $aidata['tables'];
		        foreach ($tables as $tableName=>$value) {
		        	$columns = $aidata['tables'][$tableName]['columns'];
		                foreach ($columns as $columnName=>$value) {
		                	$tablecolumn = $tableName.'.'.$columnName;
        				$formcode .= "<option value=\"".$tablecolumn."\">".$tablecolumn."</option>";
        			}
        		}
        	$formcode .= "</select>";
        	$formcode .= '</div>';
        	
        	$formcode .= '<div class="col-2">';
        	$formcode .= '<button type="button" class="btn btn-primary" onclick="submitAddRelationship()">Add</button>';
        	$formcode .= '</div>';
        	$formcode .= '</div>';
       
    $formcode .= '</form>';
    
    	echo($formcode);
    	exit;
    }
    
    
    
    if (isset($_GET['action']) && $_GET['action'] == "addtraining") {
    	
        $addtraining = array();
        
        if (isset($_POST['addtrainingmessage']) && strlen($_POST['addtrainingmessage']) > 0 && isset($_POST['addtrainingresponse']) && strlen($_POST['addtrainingresponse']) > 0) {
        	
        	$addtraining['message'] = stripslashes($_POST['addtrainingmessage']);
        	$addtraining['response'] = stripslashes($_POST['addtrainingresponse']);
        	
        	if (isset($_POST['addtrainingexampleshow']) && $_POST['addtrainingexampleshow'] > 0) {
        		$addtraining['exampleshow'] = 1;
        		$addtraining['exampletitle'] = stripslashes($_POST['addtrainingexampletitle']);
        	} else {
        		$addtraining['exampleshow'] = 0;
        		$addtraining['exampletitle'] = '';
        	}
        	
        	$aidata['training'][] = $addtraining;
        	
        	$aidata = json_encode($aidata);
        	lfmsql_query("UPDATE `geniuxs_ai` SET aidata='".addslashes($aidata)."' WHERE id='".$aiid."'") or die(lfmsql_error());
        	
        }
        
        echo("Success");
	exit;
    }
    
    if (isset($_GET['action']) && $_GET['action'] == "edittraining" && isset($_GET['trainingid']) && is_numeric($_GET['trainingid'])) {
    	
    	$trainingindex = $_GET['trainingid'];
    	
        if (isset($_POST['edittrainingmessage_'.$trainingindex]) && strlen($_POST['edittrainingmessage_'.$trainingindex]) > 0 && isset($_POST['edittrainingresponse_'.$trainingindex]) && strlen($_POST['edittrainingresponse_'.$trainingindex]) > 0 && isset($aidata['training'][$trainingindex])) {
       		
       		$aidata['training'][$trainingindex]['message'] = stripslashes($_POST['edittrainingmessage_'.$trainingindex]);
        	$aidata['training'][$trainingindex]['response'] = stripslashes($_POST['edittrainingresponse_'.$trainingindex]);
       		
       		if (isset($_POST['edittrainingexampleshow_'.$trainingindex]) && $_POST['edittrainingexampleshow_'.$trainingindex] > 0) {
       			$aidata['training'][$trainingindex]['exampleshow'] = 1;
        		$aidata['training'][$trainingindex]['exampletitle'] = stripslashes($_POST['edittrainingexampletitle_'.$trainingindex]);
       		} else {
       			$aidata['training'][$trainingindex]['exampleshow'] = 0;
       			$aidata['training'][$trainingindex]['exampletitle'] = '';
        	}
       		
       		$aidata = json_encode($aidata);
	        lfmsql_query("UPDATE `geniuxs_ai` SET aidata='".addslashes($aidata)."' WHERE id='".$aiid."'") or die(lfmsql_error());
        }
        
        echo("Success");
	exit;
    }
    
    if (isset($_GET['action']) && $_GET['action'] == "deletetraining" && isset($_GET['trainingid']) && is_numeric($_GET['trainingid'])) {
    	
        if (isset($aidata['training'][$_GET['trainingid']])) {
	        unset($aidata['training'][$_GET['trainingid']]);
        	$aidata['training'] = array_values($aidata['training']);
        	$aidata = json_encode($aidata);
        	lfmsql_query("UPDATE `geniuxs_ai` SET aidata='".addslashes($aidata)."' WHERE id='".$aiid."'") or die(lfmsql_error());
        }
        
        echo("Success");
	exit;
    }
    
    if (isset($_GET['action']) && $_GET['action'] == "loadtraining") {
    	
    	$formcode = '<form name="trainingform" method="POST">
        <h2>Add Training Examples:</h2>
        <p>Here you can train your AI to learn some of the requests that your users might make, and how to find that information in the database. Enter a message that a user might ask your AI, and for the response enter a MySQL SELECT statement that answers the question. Be sure to click the "Test" button to check that your MySQL statement works and that it loads the correct information. You can also check the box to display that particular question on the examples list of things your users can ask your AI.</p>
        <p>An AI can learn a lot simply by looking at your table/column list and descriptions, so you don\'t need to train your AI on every possible question a user might ask. But if your AI is returning a lot of incorrect responses on a particular topic, you might need to provide multiple examples and SQL queries related to that topic.</p>
        ';
        
        foreach ($aidata['training'] as $trainingindex=>$value) {
        	
        	$formcode .= "<div class=\"form-group row\">
        	<label class=\"col-sm-2 col-form-label\" for='edittrainingmessage_".$trainingindex."'><b>Message:</b></label>
        	<div class=\"col-sm-10\">
        		<input class='form-control geniuxs_create_edittraining' type='text' id='edittrainingmessage_".$trainingindex."' name='edittrainingmessage_".$trainingindex."' value=\"".htmlentities($aidata['training'][$trainingindex]['message'])."\">
        	</div>
        	</div>";
        	
        	$formcode .= "<div class=\"form-group row\">
        	<label class=\"col-sm-2 col-form-label\" for='edittrainingresponse_".$trainingindex."'><b>Response:</b></label>
        	<div class=\"col-sm-9\">
        		<input class='form-control geniuxs_create_edittraining' type='text' id='edittrainingresponse_".$trainingindex."' name='edittrainingresponse_".$trainingindex."' value=\"".htmlentities($aidata['training'][$trainingindex]['response'])."\">
        	</div>
        	<span class=\"col-sm btn btn-primary btn-sm\" onClick=\"testTraining('edittrainingresponse_".$trainingindex."');\">Test</span>
        	</div>";
        	
        	if ($aidata['training'][$trainingindex]['exampleshow'] > 0) {
        		$checkboxchecked = ' checked';
        	} else {
        		$checkboxchecked = '';
        	}
        	
        	$formcode .= "<div class=\"form-group row\">";
        	$formcode .= "<div class=\"col-sm-2\">
        	<input type='checkbox' id='edittrainingexampleshow_".$trainingindex."' name='edittrainingexampleshow_".$trainingindex."' value='1' class='geniuxs_edittraining_checkbox'".$checkboxchecked.">
        	<label class=\"col-form-label\" for='edittrainingexampleshow_".$trainingindex."'><b>Show Example:</b></label>
        	</div>
        	<div class=\"col-sm-10\">
        		<input class='form-control geniuxs_create_edittraining' type='text' id='edittrainingexampletitle_".$trainingindex."' name='edittrainingexampletitle_".$trainingindex."' value=\"".htmlentities($aidata['training'][$trainingindex]['exampletitle'])."\">
        	</div>
        	</div>";
        	
        	
        	$formcode .= '<button type="button" class="btn btn-primary" onclick="submitEditTraining('.$trainingindex.')">Save</button>';
        	
        	$formcode .= '<button type="button" class="btn btn-danger" onclick="deleteTraining('.$trainingindex.')">Delete</button>';
        	
        	$formcode .= '<hr>';
        }
	
	$formcode .= "<div class=\"form-group row\">
        	<label class=\"col-sm-2 col-form-label\" for='addtrainingmessage'><b>Message:</b></label>
        	<div class=\"col-sm-10\">
        		<input class='form-control geniuxs_create_addtraining' type='text' id='addtrainingmessage' name='addtrainingmessage' value=''>
        	</div>
        	</div>";
        	
        	$formcode .= "<div class=\"form-group row\">
        	<label class=\"col-sm-2 col-form-label\" for='addtrainingresponse'><b>Response:</b></label>
        	<div class=\"col-sm-9\">
        		<input class='form-control geniuxs_create_addtraining' type='text' id='addtrainingresponse' name='addtrainingresponse' value=''>
        	</div>
        	<span class=\"col-sm btn btn-primary btn-sm\" onClick=\"testTraining('addtrainingresponse');\">Test</span>
        	</div>";
        	
        	$formcode .= "<div class=\"form-group row\">";
        	$formcode .= "<div class=\"col-sm-2\">
        	<input type='checkbox' id='addtrainingexampleshow' name='addtrainingexampleshow' value='1' class='geniuxs_addtraining_checkbox'>
        	<label class=\"col-form-label\" for='addtrainingexampleshow'><b>Show Example:</b></label>
        	</div>
        	<div class=\"col-sm-10\">
        		<input class='form-control geniuxs_create_addtraining' type='text' id='addtrainingexampletitle' name='addtrainingexampletitle' value=''>
        	</div>
        	</div>";
        	
        	
        	$formcode .= '<button type="button" class="btn btn-primary" onclick="submitAddTraining()">Add</button>';
       
    $formcode .= '</form>';
    
    	echo($formcode);
    	exit;
    }
    
?>