<?php
// ////////////////////////////////////////////////////////////
// LFMTE v2.54
// Copyright 2006-2022 Logiscape LLC. All Rights Reserved
// http://themembershipscript.com
//
// The LFM Script ("the script") is copyrighted to
// Logiscape LLC.  Individual files and libraries
// included with the script may be governed by their own
// license agreements, which can be found in the source
// codes of those files when applicable.
//
// The sale, duplication or transfer of the script outside the
// license agreement is strictly prohibited.
//
// Any alteration of the script source code or accompanying
// materials will void any responsibility that Logiscape
// has regarding the proper functioning of the script.
// By using this script you agree to the terms and conditions
// of use of the script.
// ////////////////////////////////////////////////////////////

// -- Graph Functions --
// VisuGraph JS 1.0
// 2012-2020 Josh Abbott, http://logiscape.com

require_once "../../inc/filter.php";
require_once "../../inc/funcs.php";

include "../../inc/checkauth.php";

if(!isset($_SESSION["adminid"])) { exit; };

function countdays($firstdate, $lastdate) {
	$currdate = $firstdate;
	$daycount = 0;
	while ($currdate <= $lastdate) {
		$daycount++;
		$currdate = strftime("%Y-%m-%d", strtotime("$currdate + 1 day"));
	}
	return $daycount;
}

date_default_timezone_set('UTC');

// Prepare Graph

$datatype = $_GET['datatype'];

// Data Type 1 = Hits Delivered
// Data Type 2 = Credits Earned
// Data Type 3 = Members Surfed
// Data Type 4 = Per URL
// Data Type 5 = Per Banner
// Data Type 6 = Per Text Ad
// Data Type 7 = New Signups

// Special Data Types For Plugins
// Data Type 8 = Per Square Banner
// Data Type 9 = Per Mini Brander
// Data Type 10 = Per Peel Ad

// Data Type 15 = Per Widget Ad

if ($datatype == "" || !is_numeric($datatype) || $datatype < 1 || $datatype > 15) {
	// Set to default
	$datatype = 1;
}

if ($datatype == 1) {
	$searchvalue = "SUM(hitsdelivered) AS hits";
	$datalabel = "Hits Delivered";
} elseif ($datatype == 2) {
	$searchvalue = "SUM(crdsearned) AS hits";
	$datalabel = "Credits Earned";
} elseif ($datatype == 3) {
	$searchvalue = "SUM(memsurfed) AS hits";
	$datalabel = "Members Surfed";
} elseif ($datatype == 4) {
	$searchvalue = "SUM(perurl) AS hits";
	$datalabel = "Hits Per URL";
} elseif ($datatype == 5) {
	$searchvalue = "SUM(perbanner) AS hits";
	$datalabel = "Imps Per Banner";
} elseif ($datatype == 6) {
	$searchvalue = "SUM(pertext) AS hits";
	$datalabel = "Imps Per Text Ad";
} elseif ($datatype == 7) {
	$searchvalue = "SUM(signups) AS hits";
	$datalabel = "New Signups";
} elseif ($datatype == 8) {
	$searchvalue = "SUM(persqban) AS hits";
	$datalabel = "Imps Per Sq Banner";
} elseif ($datatype == 9) {
	$searchvalue = "SUM(permbrand) AS hits";
	$datalabel = "Imps Per Mini Bander";
} elseif ($datatype == 10) {
	$searchvalue = "SUM(perpeel) AS hits";
	$datalabel = "Imps Per Peel Ad";
} elseif ($datatype == 15) {
	$searchvalue = "SUM(perwidgetad) AS hits";
	$datalabel = "Imps Per Widget Ad";
}

// Check for extra conditions

$extracondition = "";

if (strlen($extracondition) > 0) {
	$extracondition = $extracondition." AND ";
}

if (strlen($extracondition) > 0) {
	$extraand = " AND ".$extracondition;
} else {
	$extraand = "";
}

// Pull the items list and graph over a date range

$starttime = $_GET['starttime'];
$endtime = $_GET['endtime'];

if (!is_numeric($starttime) || $starttime < 1) { exit; }
if (!is_numeric($endtime) || $endtime < 1) { exit; }

$startdate = date("Y-m-d",$starttime);
$enddate = date("Y-m-d",$endtime);

// Check The Date Range Available
$currentdate = date("Y-m-d");

$getfirstdate = lfmsql_query("Select date from `".$prefix."datestats` order by date asc limit 1");
if (lfmsql_num_rows($getfirstdate) > 0) {
	$firstavaildate = lfmsql_result($getfirstdate, 0, "date");
} else {
	$firstavaildate = strftime("%Y-%m-%d", strtotime("$currentdate + 30 days ago"));
}

$getlastdate = lfmsql_query("Select date from `".$prefix."datestats` order by date desc limit 1");
if (lfmsql_num_rows($getlastdate) > 0) {
	$lastavaildate = lfmsql_result($getlastdate, 0, "date");
} else {
	$lastavaildate = $currentdate;
}

$numdays = countdays($startdate, $enddate);
$numelems = 0;

if ($numdays > 10950) {
	// Prevents loading date ranges over 30 years
	$startdate = date("Y-m-d", strtotime("-7 days"));
	$enddate = date("Y-m-d", strtotime("-1 day"));
	$numdays = countdays($startdate, $enddate);
}

if ($numdays > 1000) {
	//Switch to yearly view
	
	$graphtype = "yearly";
	
	$yearstartdate = strftime("%Y", strtotime($startdate));
	$yearenddate = strftime("%Y", strtotime($enddate));

	$numelems = 0;

	$getdays = lfmsql_query("Select ".$searchvalue.", DATE_FORMAT(date, '%Y') AS dateyear from `".$prefix."datestats` WHERE date >= '".$yearstartdate."-01-01' and date <= '".$yearenddate."-12-31'".$extraand." GROUP BY DATE_FORMAT(date, '%Y') ORDER BY dateyear ASC");
	
	$i = 0;
	while ($row = lfmsql_fetch_array($getdays)) {
		$thedate = $row["dateyear"];
		$datelist[$i] = $thedate;
		$sumlist[$i] = $row["hits"];
		$i++;
	}
	if (!is_array($datelist) || count($datelist) < 1) {
		// Set Empty Date Array
		$datelist[0] = 0;
	}
	
	$currdate = $yearstartdate;
	$i = 0;
	while ($currdate <= $yearenddate) {
		$numelems++;
		$dateposition = array_search($currdate, $datelist);
		if ($dateposition === false) {
			//No Stats For Date
			$thetime = mktime(0,0,0,01,1,substr($currdate,0,4));
			$statsum[$i] = 0;
			$xname[$i] = date("Y", $thetime);
			$xname[$i] = $xname[$i]."<br><br>";
			$xvals[$i] = $thetime * 1000;
		} else {
			//Enter Stats For Date
			$thetime = mktime(0,0,0,01,1,substr($currdate,0,4));
			$statsum[$i] = $sumlist[$dateposition];
			$xname[$i] = date("Y", $thetime);
			$xname[$i] = $xname[$i]."<br><br>";
			$xvals[$i] = $thetime * 1000;
		}
	
		$currdate = $currdate+1;
		$i++;
	}
	
} elseif ($numdays > 40) {
	//Switch to monthly view
	
	$graphtype = "monthly";
	
	$monthstartdate = strftime("%Y-%m", strtotime($startdate));
	$monthenddate = strftime("%Y-%m", strtotime($enddate));
	$monthlastday = date('t', strtotime($enddate));

	$currdate = $startdate;
	
	while ($currdate <= $enddate) {
	
	$selecteddate = mktime(0,0,0,substr($currdate,5,2),substr($currdate,8),substr($currdate,0,4));
	
	$selectedmonth = date("m", $selecteddate);
	$selectedyear = date("Y", $selecteddate);
	$daya = $selectedyear."-".$selectedmonth."-"."01";
	$dayb = $selectedyear."-".$selectedmonth."-"."31";
	$monthname = date("M", $selecteddate);
		
	if ($monthname == "Jan") {
		$monthname = "Jan<br>".$selectedyear;
	} else {
		$monthname = $monthname."<br><br>";
	}
	
	$getmonth = lfmsql_query("Select ".$searchvalue." from `".$prefix."datestats` WHERE date >= '".$daya."' AND date <= '".$dayb."'".$extraand);
	$currentsum = lfmsql_result($getmonth, 0, "hits");
	
	if ($currentsum == NULL) {
		$currentsum = 0;
	}

	$statsum[$numelems] = $currentsum;
	$xname[$numelems] = $monthname;
	$xvals[$numelems] = mktime(0,0,0,substr($daya,5,2),2,substr($daya,0,4)) * 1000;
	$numelems = $numelems+1;
	
	$currdate = strftime("%Y-%m-%d", strtotime("$daya + 1 month"));
	
	}
	
	
	if ($selectedmonth != date("m", strtotime($enddate))) {
		// Run the last month
		
		$selecteddate = mktime(0,0,0,substr($currdate,5,2),substr($currdate,8),substr($currdate,0,4));
		
		$selectedmonth = date("m", $selecteddate);
		$selectedyear = date("Y", $selecteddate);
		$daya = $selectedyear."-".$selectedmonth."-"."01";
		$dayb = $selectedyear."-".$selectedmonth."-"."31";
		$monthname = date("M", $selecteddate);
			
		if ($monthname == "Jan") {
			$monthname = "Jan<br>".$selectedyear;
		} else {
			$monthname = $monthname."<br><br>";
		}
		
		$getmonth = lfmsql_query("Select ".$searchvalue." from `".$prefix."datestats` WHERE date >= '".$daya."' AND date <= '".$dayb."'".$extraand);
		$currentsum = lfmsql_result($getmonth, 0, "hits");
		
		if ($currentsum == NULL) {
			$currentsum = 0;
		}
		
		$statsum[$numelems] = $currentsum;
		$xname[$numelems] = $monthname;
		$xvals[$numelems] = mktime(0,0,0,substr($daya,5,2),2,substr($daya,0,4)) * 1000;
		$numelems = $numelems+1;
	}
	

} else {
	// Switch to daily view
	
	$graphtype = "daily";

	$numelems = $numdays;

	$getdays = lfmsql_query("Select ".$searchvalue.", date from `".$prefix."datestats` WHERE date >= '".$startdate."' and date <= '".$enddate."'".$extraand." GROUP BY date ORDER BY date ASC");
	
	$i = 0;
	while ($row = lfmsql_fetch_array($getdays)) {
		$thedate = $row["date"];
		$datelist[$i] = $thedate;
		$sumlist[$i] = $row["hits"];
		$i++;
	}
	if (!is_array($datelist) || count($datelist) < 1) {
		// Set Empty Date Array
		$datelist[0] = 0;
	}
	
	$currdate = $startdate;
	$i = 0;
	while ($currdate <= $enddate) {
	
		$dateposition = array_search($currdate, $datelist);
		if ($dateposition === false) {
			//No Stats For Date
			$thetime = mktime(0,0,0,substr($currdate,5,2),substr($currdate,8),substr($currdate,0,4));
			$statsum[$i] = 0;
			$xname[$i] = date("d", $thetime);
			if ($xname[$i] == 1) { $xname[$i] = $xname[$i]."<br>".date("M", $thetime); } else { $xname[$i] = $xname[$i]."<br><br>"; }
			$xvals[$i] = $thetime * 1000;
		} else {
			//Enter Stats For Date
			$thetime = mktime(0,0,0,substr($currdate,5,2),substr($currdate,8),substr($currdate,0,4));
			$statsum[$i] = $sumlist[$dateposition];
			$xname[$i] = date("d", $thetime);
			if ($xname[$i] == 1) { $xname[$i] = $xname[$i]."<br>".date("M", $thetime); } else { $xname[$i] = $xname[$i]."<br><br>"; }
			$xvals[$i] = $thetime * 1000;
		}
	
		$currdate = strftime("%Y-%m-%d", strtotime("$currdate + 1 day"));
		$i++;
	}
	
}

// Check Zoom In
if ($numdays > 7) {
	$canzoomin = "yes";
} else {
	$canzoomin = "no";
}

// Check Zoom Out
if ($numdays >= 7300) {
	// Show A Max Of 20 Years
	$canzoomout = "no";
} elseif ($firstavaildate <= $startdate || $lastavaildate >= $enddate) {
	$canzoomout = "yes";
} else {
	$canzoomout = "no";
}

// Check Pan Left
if ($firstavaildate <= $startdate) {
	$canpanleft = "yes";
} else {
	$canpanleft = "no";
}

// Check Pan Right
if ($lastavaildate >= $enddate) {
	$canpanright = "yes";
} else {
	$canpanright = "no";
}


// End Prepare Graph

if ($numelems < 1) {
	// No Data To Display
	exit;
}

// Make Data Array And Ticks (Axis Labels)
$datatext = $xvals[0].",".$statsum[0];
$tickstext = $xvals[0].",".$xname[0];
for ($i = 1; $i < $numelems; $i++) {
	$datatext .= "|".$xvals[$i].",".$statsum[$i];
	$tickstext .= "|".$xvals[$i].",".$xname[$i];			
}

// Set The Top Line On The Y Axis By Finding The Highest Stat
$maxstat = max($statsum);
$maxstatstr = (string) $maxstat;
$maxstatlen = strlen($maxstatstr);
$maxstatfirst = $maxstatstr[0];
$maxstatdigits = pow(10, $maxstatlen-1);
$maxstat = ($maxstatfirst+1)*$maxstatdigits;

// Output The Data
echo("#STARTSTARTDATE#".$startdate."#ENDSTARTDATE#");
echo("#STARTENDDATE#".$enddate."#ENDENDDATE#");

echo("#STARTGRAPHTYPE#".$graphtype."#ENDGRAPHTYPE#");
echo("#STARTDATALABEL#".$datalabel."#ENDDATALABEL#");

echo("#STARTDATA#".$datatext."#ENDDATA#");
echo("#STARTTICKS#".$tickstext."#ENDTICKS#");
echo("#STARTMAX#".$maxstat."#ENDMAX#");

echo("#STARTZOOMIN#".$canzoomin."#ENDZOOMIN#");
echo("#STARTZOOMOUT#".$canzoomout."#ENDZOOMOUT#");
echo("#STARTPANLEFT#".$canpanleft."#ENDPANLEFT#");
echo("#STARTPANRIGHT#".$canpanright."#ENDPANRIGHT#");

exit;

?>