<?php
// Get the current date and calculate the timestamps for the last 2 days, today, and the next 2 days
$currenttime = time();
$today = strtotime('today');
$two_days_ago = strtotime('-2 days', $today);
$two_days_later = strtotime('+2 days', $today);

// Query to get promos for the last 2 days, today, and the next 2 days
$query = "SELECT p.id, p.starttime, p.endtime, s.domain, COUNT(DISTINCT a.usrid) as claimed
          FROM oto_active_surfer_promos p
          LEFT JOIN oto_active_surfer_sites s ON p.id = s.promoid
          LEFT JOIN oto_active_surfers a ON p.id = a.promoid
          WHERE p.active>0 AND p.starttime BETWEEN $two_days_ago AND $two_days_later
          GROUP BY p.id, p.starttime, p.endtime, s.domain
          ORDER BY p.starttime ASC";

$result = lfmsql_query($query);

// Initialize arrays to store promo data
$past_promos = [];
$current_promos = [];
$future_promos = [];

// Process the query results
while ($row = lfmsql_fetch_assoc($result)) {
    $promo_date = date('M j, Y', $row['starttime']); // Format date as 'Aug 7, 2024'
    $promo_id = $row['id'];
    $exchange = $row['domain'];

    if ($row['endtime'] <= $currenttime) {
        $past_promos[$promo_id]['id'] = $promo_id;
        $past_promos[$promo_id]['date'] = $promo_date;
        $past_promos[$promo_id]['exchanges'][$exchange] = $exchange; // Use associative array to avoid duplicates
        $past_promos[$promo_id]['claimed'] = $row['claimed'];
    } elseif ($row['starttime'] < $currenttime && $row['endtime'] > $currenttime) {
        $current_promos[$promo_id]['id'] = $promo_id;
        $current_promos[$promo_id]['date'] = $promo_date;
        $current_promos[$promo_id]['exchanges'][$exchange] = $exchange;
        $current_promos[$promo_id]['claimed'] = $row['claimed'];
    } elseif ($row['starttime'] >= $currenttime) {
        $future_promos[$promo_id]['id'] = $promo_id;
        $future_promos[$promo_id]['date'] = $promo_date;
        $future_promos[$promo_id]['exchanges'][$exchange] = $exchange;
    }
}

// Start HTML output
?>
<style>
    .promo-widget-no-underline:hover, .promo-widget-no-underline:focus {
        text-decoration: none;
    }
    .promo-widget-rotate {
        transition: transform 0.2s ease-in-out;
    }
    .promo-widget-rotate.down {
        transform: rotate(90deg);
    }
</style>
<script>
    document.addEventListener('DOMContentLoaded', function() {
        const promoWidgetSections = ['recentPromos', 'currentPromos', 'upcomingPromos'];

        promoWidgetSections.forEach(section => {
            const element = document.getElementById(section);
            const button = document.querySelector(`[data-target="#${section}"] i.promo-widget-arrow`);

            // Load saved state from localStorage
            const isExpanded = localStorage.getItem(section) === 'true';
            if (isExpanded) {
                element.classList.add('show');
                button.classList.add('down');
            } else {
                element.classList.remove('show');
                button.classList.remove('down');
            }

            // Add event listener to toggle icon and save state
            document.querySelector(`[data-target="#${section}"]`).addEventListener('click', function() {
                const isCurrentlyExpanded = element.classList.contains('show');
                localStorage.setItem(section, !isCurrentlyExpanded);
                button.classList.toggle('down', !isCurrentlyExpanded);
            });
        });
    });
</script>
<!-- Start Widget Output -->
<div class="col-12 col-lg-6 col-xl-4">
    <div class="card">
        <div class="card-header">
            <span class="lfm_widget_title">Cross Promos Overview</span>
        </div>
        <div class="card-body">
            <div class="mb-3">
                <button class="btn btn-link text-secondary d-flex justify-content-between align-items-center w-100 text-left promo-widget-no-underline" data-toggle="collapse" data-target="#recentPromos" aria-expanded="false" aria-controls="recentPromos">
                    <span><i class="fas fa-history"></i> Recent Promos</span>
                    <i class="fas fa-chevron-right promo-widget-arrow promo-widget-rotate"></i>
                </button>
                <div id="recentPromos" class="collapse">
                    <ul class="list-group mb-3">
                        <?php if (empty($past_promos)) { ?>
                            <li class="list-group-item">There were no promos in the last 2 days</li>
                        <?php } else { 
                            foreach ($past_promos as $promo) { ?>
                                <li class="list-group-item">
                                    <strong><i class="fas fa-calendar-day"></i> Date:</strong> <?php echo $promo['date']; ?>
                                    <br><strong><i class="fas fa-exchange-alt"></i> Exchanges:</strong> <?php echo implode(', ', $promo['exchanges']); ?>
                                    <br><strong><i class="fas fa-users"></i> Claimed:</strong> <a target="_blank" href="activesurfers_rewards.php?promoid=<?php echo $promo['id']; ?>"><?php echo $promo['claimed']; ?></a>
                                </li>
                            <?php }
                        } ?>
                    </ul>
                </div>
            </div>
            <div class="mb-3">
                <button class="btn btn-link text-secondary d-flex justify-content-between align-items-center w-100 text-left promo-widget-no-underline" data-toggle="collapse" data-target="#currentPromos" aria-expanded="false" aria-controls="currentPromos">
                    <span><i class="fas fa-calendar-day"></i> Current Promos</span>
                    <i class="fas fa-chevron-right promo-widget-arrow promo-widget-rotate"></i>
                </button>
                <div id="currentPromos" class="collapse">
                    <ul class="list-group mb-3">
                        <?php if (empty($current_promos)) { ?>
                            <li class="list-group-item">There are no promos running today</li>
                        <?php } else { 
                            foreach ($current_promos as $promo) { ?>
                                <li class="list-group-item">
                                    <strong><i class="fas fa-calendar-day"></i> Date:</strong> <?php echo $promo['date']; ?>
                                    <br><strong><i class="fas fa-exchange-alt"></i> Exchanges:</strong> <?php echo implode(', ', $promo['exchanges']); ?>
                                    <br><strong><i class="fas fa-users"></i> Claimed:</strong> <a target="_blank" href="activesurfers_rewards.php?promoid=<?php echo $promo['id']; ?>"><?php echo $promo['claimed']; ?></a>
                                </li>
                            <?php }
                        } ?>
                    </ul>
                </div>
            </div>
            <div>
                <button class="btn btn-link text-secondary d-flex justify-content-between align-items-center w-100 text-left promo-widget-no-underline" data-toggle="collapse" data-target="#upcomingPromos" aria-expanded="false" aria-controls="upcomingPromos">
                    <span><i class="fas fa-calendar-plus"></i> Upcoming Promos</span>
                    <i class="fas fa-chevron-right promo-widget-arrow promo-widget-rotate"></i>
                </button>
                <div id="upcomingPromos" class="collapse">
                    <ul class="list-group">
                        <?php if (empty($future_promos)) { ?>
                            <li class="list-group-item">There are no promos for the next 2 days</li>
                        <?php } else { 
                            foreach ($future_promos as $promo) { ?>
                                <li class="list-group-item">
                                    <strong><i class="fas fa-calendar-day"></i> Date:</strong> <?php echo $promo['date']; ?>
                                    <br><strong><i class="fas fa-exchange-alt"></i> Exchanges:</strong> <?php echo implode(', ', $promo['exchanges']); ?>
                                </li>
                            <?php }
                        } ?>
                    </ul>
                </div>
            </div>
        </div>
    </div>
</div>
<!-- End Widget Output -->