<?php

// LFM Authorize.net Integration v1.0
// Written By Josh Abbott

// Handles ajax calls from the purchase confirmation modals

@session_start();

require_once "inc/filter.php";
require_once "inc/funcs.php";

include "inc/config.php";
lfmsql_connect($dbhost,$dbuser,$dbpass);
lfmsql_select_db($dbname) or die( "Unable to select database");

include_once "inc/lfm_password.php";
include_once "inc/sql_funcs.php";

if (!isset($_POST['authnetItemId']) || !is_numeric($_POST['authnetItemId']) || $_POST['authnetItemId'] < 1) {
	echo("Invalid Item ID");
	exit;
}

if (isset($_SESSION["adminid"])) {
	echo('You must logout of the Admin Panel to make a purchase.');
	exit;
}

if (isset($_POST["login"]) && strlen($_POST["login"]) > 0 && isset($_POST["password"]) && strlen($_POST["password"]) > 0) {
	
	// User Is Logging In
	
	// Check Banned IP
	if (bannedIp(getip()) !== false) { echo("Your IP Is Banned"); exit; }
	 
	 // Sanitize input values
	 $s_login=sanitize_sql($_POST["login"]);
	 $s_password=sanitize_sql($_POST["password"]);
	 
	$max_fail_name = lfmsql_result(lfmsql_query("SELECT max_fail_name FROM `".$prefix."settings`"), 0);
	$max_fail_ip = lfmsql_result(lfmsql_query("SELECT max_fail_ip FROM `".$prefix."settings`"), 0);
	$tenminago = time() - 600;
	
	// IP Address Brute Force Protection
	if (isset($max_fail_ip) && is_numeric($max_fail_ip) && $max_fail_ip > 0) {
		$countfailip = lfmsql_result(lfmsql_query("SELECT COUNT(*) FROM `".$prefix."login_log` WHERE logtime >= ".$tenminago." AND ipaddress='".$_SERVER['REMOTE_ADDR']."' AND invalidlogin=1"), 0);
		if ($countfailip >= $max_fail_ip) {
			echo("As a security precaution, you have been temporary blocked due to too many invalid login attempts.  This block will be automatically lifted within the next hour.");
			exit;
		}
	}
	
	 /* Try to authenticate this user against db */
	 $authquery="SELECT Id, username, email, password, status FROM ".$prefix."members WHERE username='".$s_login."'";
	 $authresult=@lfmsql_query($authquery);
	 $num=@lfmsql_num_rows($authresult);
	 if($num <= 0) {
		// Username doesn't exist
		@lfmsql_query("INSERT INTO `".$prefix."login_log` (logtime, userid, ipaddress, invalidlogin) VALUES ('".time()."', -1, '".$_SERVER['REMOTE_ADDR']."', 1)");
		$authenticated=0;
		$_SESSION["uid"]="";
		$_SESSION["user_password"]="";
		$_SESSION["userid"]="";
		if (!isset($_SESSION["adminid"])){
			@session_destroy();
		}
		echo("Invalid Username Or Password");
		exit;
	 } else	{
		$authrow=@lfmsql_fetch_array($authresult);
		
		// Validate Password
		$loginuserid = $authrow["Id"];
		$passwordhash = $authrow["password"];
		
		$verifiedip = lfmsql_result(lfmsql_query("SELECT COUNT(*) FROM `".$prefix."login_log` WHERE userid=".$loginuserid." AND ipaddress='".$_SERVER['REMOTE_ADDR']."' AND invalidlogin=0"), 0);
		
		if ($verifiedip < 1) {
			// Username Brute Force Protection
			if (isset($max_fail_name) && is_numeric($max_fail_name) && $max_fail_name > 0) {
				$countfaillogins = lfmsql_result(lfmsql_query("SELECT COUNT(*) FROM `".$prefix."login_log` WHERE logtime >= ".$tenminago." AND userid=".$loginuserid." AND invalidlogin=1"), 0);
				if ($countfaillogins >= $max_fail_name) {
					echo("As a security precaution, you have been temporary blocked due to too many invalid login attempts.  This block will be automatically lifted within the next hour.");
					exit;
				}
			}
		}
		
		if (strlen($passwordhash) == 32) {
			// Check Using Old MD5 Format
			if ($passwordhash == md5($s_password)) {
				// Correct Password
				@lfmsql_query("INSERT INTO `".$prefix."login_log` (logtime, userid, ipaddress, invalidlogin) VALUES ('".time()."', ".$loginuserid.", '".$_SERVER['REMOTE_ADDR']."', 0)");
				$passwordhash = lfmMakePassword($s_password);
				lfmsql_query("UPDATE ".$prefix."members SET password='".$passwordhash."' WHERE Id='".$loginuserid."'") or die(lfmsql_error());
			} else {
				// Wrong Password
				@lfmsql_query("INSERT INTO `".$prefix."login_log` (logtime, userid, ipaddress, invalidlogin) VALUES ('".time()."', ".$loginuserid.", '".$_SERVER['REMOTE_ADDR']."', 1)");
				$authenticated=0;
				$_SESSION["uid"]="";
				$_SESSION["user_password"]="";
				$_SESSION["userid"]="";
				if (!isset($_SESSION["adminid"])){
					@session_destroy();
				}
				echo("Invalid Username Or Password");
				exit;
			}
		} else {
			// Check Using New Format
			if (lfmCheckPassword($s_password, $passwordhash)) {
				// Correct Password
				@lfmsql_query("INSERT INTO `".$prefix."login_log` (logtime, userid, ipaddress, invalidlogin) VALUES ('".time()."', ".$loginuserid.", '".$_SERVER['REMOTE_ADDR']."', 0)");
			} else {
				// Wrong Password
				@lfmsql_query("INSERT INTO `".$prefix."login_log` (logtime, userid, ipaddress, invalidlogin) VALUES ('".time()."', ".$loginuserid.", '".$_SERVER['REMOTE_ADDR']."', 1)");
				$authenticated=0;
				$_SESSION["uid"]="";
				$_SESSION["user_password"]="";
				$_SESSION["userid"]="";
				if (!isset($_SESSION["adminid"])){
					@session_destroy();
				}
				echo("Invalid Username Or Password");
				exit;
			}
		}
		
		// Delete Old Log Entries
		$monthago = time() - 2678400;
		@lfmsql_query("DELETE FROM `".$prefix."login_log` WHERE logtime < ".$monthago);
		
		// Check Banned Email
		if (bannedEmail($authrow["email"]) !== false) { echo("Your Email Is Banned"); exit; }
		
		// Authenticated
		$logintime=date("Y-m-d H:i:s");
		$ip=getip();
		if (!isset($_SESSION["adminid"])){
			@lfmsql_query("UPDATE ".$prefix."members SET lastip='$ip', lastlogin='$logintime', numLogins=numLogins+1 WHERE Id=".$authrow["Id"]) or die(lfmsql_error());
		}
		$_SESSION["uid"] = $authrow["username"];
		$_SESSION["user_password"] = $passwordhash;
		$_SESSION["userid"] = $authrow["Id"];
		setcookie("lfmpr3s5gy", session_name(), time()+3600);
		$authenticated=1;
	}
} elseif (isset($_SESSION["uid"]) && strlen($_SESSION["uid"]) > 0 && isset($_SESSION["user_password"]) && strlen($_SESSION["user_password"]) > 0 && isset($_SESSION["userid"]) && is_numeric($_SESSION["userid"]) && $_SESSION["userid"] > 0) {
	
	// User Is Logged In
	require "inc/userauth.php";
	
} else {
	
	// User Needs To Login
	echo("loginform");
	exit;
}

require_once "authnet_ipn_funcs.php";

$getitem = lfmsql_query("SELECT * FROM ".$prefix."ipn_products WHERE id='".$_POST['authnetItemId']."'") or die(lfmsql_error());
if (lfmsql_num_rows($getitem) < 1) {
	echo("Item Not Found");
	exit;
}
$iteminfo = lfmsql_fetch_array($getitem);

$itemname = $iteminfo['name'];
$itemprice = $iteminfo['amount'];

if($iteminfo['type'] != "N") {
	// Recurring Subscription
	$confirmbtn = 'Confirm Subscription';
	if($iteminfo['type']=="D") { $unit="days"; }
	if($iteminfo['type']=="M") { $unit="months"; }
	if($iteminfo['type']=="Y") { $unit="months"; $iteminfo['period']=$iteminfo['period']*12; }
	$recurring = 'Rebills every '.$iteminfo['period'].' '.$unit;
	if ($iteminfo['trial_period'] > 0 && ($iteminfo['trial_type'] == "D" || $iteminfo['trial_type'] == "M" || $iteminfo['trial_type'] == "Y")) {
		if($iteminfo['trial_type']=="D") { $startunit="days"; }
		if($iteminfo['trial_type']=="M") { $startunit="months"; }
		if($iteminfo['trial_type']=="Y") { $startunit="months"; $iteminfo['trial_period']=$iteminfo['trial_period']*12; }
		if ($iteminfo['trial_period'] == 1) { $startunit = substr($startunit, 0, -1); }
		if ($iteminfo['trial_amount'] <= 0) {
			$iteminfo['trial_amount'] = 'Free';
		} else {
			$iteminfo['trial_amount'] = '$'.$iteminfo['trial_amount'];
		}
		$recurring = $iteminfo['trial_amount'].' for '.$iteminfo['trial_period'].' '.$startunit.' then<br>'.$recurring;
	}
	if (isset($iteminfo['recurring']) && is_numeric($iteminfo['recurring']) && $iteminfo['recurring'] > 0) {
		$recurring = $recurring.' for '.$iteminfo['recurring'].' '.$unit;
	}
} else {
	// One Time Purchase
	$confirmbtn = 'Confirm Purchase';
	$recurring = 'none';
}

$authnetid = authnetGetCustomerId($_SESSION["userid"]);
$paymentmethods = authnetGetPaymentMethods($authnetid);

if ($paymentmethods !== false && strlen($paymentmethods) > 0) {
	$listpaymentmethods = $paymentmethods;
} else {
	$listpaymentmethods = '';
}

$_SESSION['authnetbuyid'] = $_POST['authnetItemId'];
$_SESSION['authnetbuytime'] = time();

if (isset($_POST['authnetItemName']) && strlen($_POST['authnetItemName']) > 0) {
	$_SESSION['authnetbuyname'] = $_POST['authnetItemName'];
}

echo($itemname.'|'.$itemprice.'|'.$confirmbtn.'|'.$recurring.'|'.$listpaymentmethods);
exit;

?>