<?php

// /////////////////////////////////////////////////////////////////////
// GeniuXs v3
// Copyright 2023 Logiscape LLC. All Rights Reserved
// https://geniuxs.com
//
// The sale, duplication or transfer of the script to any 
// person other than the original purchaser is a violation
// of the purchase agreement and is strictly prohibited.
// Any alteration of the script source code or accompanying 
// materials will void any responsibility that Logiscape LLC
// has regarding the proper functioning of the script.
// /////////////////////////////////////////////////////////////////////

// Queue Cron Job - Mails The Messages

require_once "inc/filter.php";

include "inc/config.php";
require_once "inc/funcs.php";
require_once "inc/swift/lib/swift_required.php";
lfmsql_connect($dbhost,$dbuser,$dbpass);
lfmsql_select_db($dbname) or die("Unable to select database");

function AddQueueLog($prefix, $entry) {
	// Check and remove old entries
	$countprev = lfmsql_result(lfmsql_query("SELECT COUNT(*) from geniuxs_queue_log"), 0);
	if ($countprev > 5040) {
		$num_to_del = $countprev - 5040;
		lfmsql_query("DELETE FROM geniuxs_queue_log ORDER BY logtimestamp ASC LIMIT ".$num_to_del) or die(lfmsql_error());
	}
	// Add entry
	$ts=date("Y-m-d H:i:s");
	$queuecount = lfmsql_result(lfmsql_query("SELECT COUNT(*) FROM `geniuxs_queue`"), 0);
	
	if (stripos($entry, "Sent") !== false && stripos($entry, "messages") !== false) {
		// Add the number sent this cycle
		$splitentry = explode(" ", $entry);
		$entrynumber = trim($splitentry[1]);
		if (is_numeric($entrynumber)) {
			$queuecount = $queuecount + $entrynumber;
		}
	}
	
	@lfmsql_query("INSERT INTO geniuxs_queue_log (logtimestamp, logtime, logentry, queuecount) VALUES ('".time()."', '$ts', '".$entry."', '".$queuecount."')") or die(lfmsql_error());
}

$currenttime = time();
$currentdate = date("Y-m-d");

$currenthour = date("G");
$currentmin = date("i");

// Check if cron file is called too soon

$getlastcron = lfmsql_query("Select valvalue from geniuxs_vals WHERE valtype='lastqueue'") or die(lfmsql_error());

if (lfmsql_num_rows($getlastcron) <= 0) {
	AddQueueLog($prefix, "Missing lastqueue field in geniuxs_vals");
	exit;
}

$lastcron = lfmsql_result($getlastcron, 0, "valvalue");

if ($lastcron > ($currenttime-55)) {
	AddQueueLog($prefix, "Current time is ".$currenttime.".  Last queue cron was called at ".$lastcron);
	exit;
}

@lfmsql_query("Update geniuxs_vals set valvalue='$currenttime' WHERE valtype='lastqueue'") or die(lfmsql_error());

// End check if cron file is called too soon


// Check for scheduled pauses
$getpause = lfmsql_query("SELECT durationmins FROM `geniuxs_pausesched` WHERE hour='".$currenthour."' AND minute='".$currentmin."' ORDER BY durationmins DESC LIMIT 1") or die(lfmsql_error());
if (lfmsql_num_rows($getpause) > 0) {
	// Schedule a pause
	$pausemins = lfmsql_result($getpause, 0, "durationmins");
	$pausesecs = $pausemins*60;
	$starttime = $currenttime;
	$endtime = $currenttime + $pausesecs;
	lfmsql_query("INSERT INTO `geniuxs_pausequeue` (starttime, endtime) VALUES ('".$starttime."', '".$endtime."')") or die(lfmsql_error());
}
// End check for scheduled pauses


// Delete expired pauses
lfmsql_query("DELETE FROM `geniuxs_pausequeue` WHERE endtime < '".$currenttime."'") or die(lfmsql_error());


// Check for current pauses
$checkpause = lfmsql_query("SELECT endtime FROM `geniuxs_pausequeue` WHERE starttime <= '".$currenttime."'") or die(lfmsql_error());
if (lfmsql_num_rows($checkpause) > 0) {
	$endtime = lfmsql_result($checkpause, 0, "endtime");
	$pauseleft = $currenttime - $endtime;
	AddQueueLog($prefix, "Queue paused for ".$pauseleft." more seconds.");
	exit;
}
// End check for current pauses


// Check if the cron is still running

$getlastmail = lfmsql_query("Select valvalue from geniuxs_vals WHERE valtype='lastmail'") or die(lfmsql_error());

if (lfmsql_num_rows($getlastmail) <= 0) {
	AddQueueLog($prefix, "Missing lastmail field in geniuxs_vals");
	exit;
}

$lastmail = lfmsql_result($getlastmail, 0, "valvalue");

if ($lastmail > ($currenttime-40)) {
	AddQueueLog($prefix, "Queue cron still running from last call");
	exit;
}

// End check if the cron is still running


// Start mailing process

// Query settings table for sitename etc
$res=@mysql_query("SELECT sitename, smtp_server, smtp_user, smtp_pass, affurl FROM ".$prefix."settings") or die("1040: Unable to find ".$prefix."settings!");
$row=@mysql_fetch_array($res);
$sitename=$row["sitename"];
$smtp_server=$row["smtp_server"];
$smtp_user=$row["smtp_user"];
$smtp_pass=$row["smtp_pass"];
$affurl=$row["affurl"];
if ($affurl != "") {
	$spliturl = explode("/", $affurl);
	$sitedomain = $spliturl[2];
	$sitedomain = str_replace("www.", "", $sitedomain);
} else {
	$sitedomain = "";
}

$getpermin = lfmsql_query("Select valvalue from geniuxs_vals WHERE valtype='messagespermin'") or die(lfmsql_error());

if (lfmsql_num_rows($getpermin) <= 0) {
	AddQueueLog($prefix, "Missing messagespermin field in geniuxs_vals");
	exit;
}

$permin = lfmsql_result($getpermin, 0, "valvalue");

$getmessages = lfmsql_query("SELECT id, type, messageid, userid, toaddress, fromname, fromaddress, linkkey FROM `geniuxs_queue` ORDER BY priority DESC, id ASC LIMIT ".$permin) or die(lfmsql_error());

if (lfmsql_num_rows($getmessages) <= 0) {
	AddQueueLog($prefix, "No messages");
	exit;
}

// Get bounce address
$rres=@lfmsql_query("SELECT bounceaddress FROM ".$prefix."settings");
{
	$rrow=@lfmsql_fetch_array($rres);
	$bounceaddress=$rrow["bounceaddress"];
}

$delivercount = 0;

if (strlen($smtp_server) < 1) {
	$smtp_server = "localhost";
}

$transport = Swift_SmtpTransport::newInstance($smtp_server, 25);

if (strlen($smtp_user) > 0 && strlen($smtp_pass) > 0) {
	// Using Username/Password
	$transport->setUsername($smtp_user);
	$transport->setPassword($smtp_pass);
}

$mailer = Swift_Mailer::newInstance($transport);
$mailer->registerPlugin(new Swift_Plugins_AntiFloodPlugin(100));

while ($queueinfo = lfmsql_fetch_assoc($getmessages)) {
	
	$queueid = $queueinfo["id"];
	$queuetype = $queueinfo["type"];
	$messageid = $queueinfo["messageid"];
	
	$toid = $queueinfo["userid"];
	$toaddress = $queueinfo["toaddress"];
	$fromname = $queueinfo["fromname"];
	$fromaddress = $queueinfo["fromaddress"];
	$linkkey = $queueinfo["linkkey"];
	
	if ($queuetype >= 1 && is_numeric($messageid) && $messageid > 0) {
		// Get sent message
		
		if ($queuetype == 1) {
			$getsent = lfmsql_query("SELECT subject, bodytext FROM `geniuxs_schedsent` WHERE messageid=".$messageid." LIMIT 1") or die(lfmsql_error());
			$messagetype = "smid";
		} elseif ($queuetype == 2) {
			$getsent = lfmsql_query("SELECT subject, bodytext FROM `geniuxs_groupmails` WHERE id=".$messageid." LIMIT 1") or die(lfmsql_error());
			$messagetype = "gmid";
		} else {
			$getsent = lfmsql_query("SELECT subject, bodytext FROM `geniuxs_newmem` WHERE id=".$messageid." LIMIT 1") or die(lfmsql_error());
			$messagetype = "nmid";
		}
		
		if (lfmsql_num_rows($getsent) > 0) {
			
			$sentinfo = lfmsql_fetch_assoc($getsent);
			$subjecttext = trim($sentinfo["subject"]);
			$bodytext = trim($sentinfo["bodytext"]);
			
			if (strlen($subjecttext) >= 1 && strlen($bodytext) >= 1) {

				$getaccemail = lfmsql_query("SELECT email FROM `".$prefix."members` WHERE Id=".$toid." LIMIT 1");
				if (lfmsql_num_rows($getaccemail) > 0) {

					$accinfo = lfmsql_fetch_assoc($getaccemail);
					$accemail = $accinfo["email"];
					$memberemail = $toaddress;
					
					$subjecttext = str_ireplace(':SPLITSUB:', '', $subjecttext);
					
					$fullmailtext = $subjecttext.':SPLITSUB:'.$bodytext;
					
					$fullmailtext = translate_site_tags($fullmailtext);
					$fullmailtext = translate_user_tags($fullmailtext, $accemail);
					
					$splitfullmailtext = explode(':SPLITSUB:', $fullmailtext);
					
					$subjecttext = $splitfullmailtext[0];
					$bodytext = $splitfullmailtext[1];
					
					if (stripos($bodytext, '<') !== FALSE && stripos($bodytext, '>') !== FALSE) {
						$mailformat = "html";
					}
					
					$clicklinkurl = $affurl."geniuxs_click.php?userid=".$toid."&".$messagetype."=".$messageid."&linkkey=".$linkkey;
					
					if (isset($mailformat) && $mailformat == "html") {
						if (stripos($bodytext, 'href=#CLICKLINK#') === FALSE && stripos($bodytext, 'href="#CLICKLINK#') === FALSE && stripos($bodytext, 'href = #CLICKLINK#') === FALSE && stripos($bodytext, 'href = "#CLICKLINK#') === FALSE) {
							$clicklinkurl = "<a target=\"_blank\" href=\"".$clicklinkurl."\">".$clicklinkurl."</a>";
						}
					}
					
					$bodytext = str_ireplace('#CLICKLINK#', $clicklinkurl, $bodytext);
					
					$unsubscribe = $affurl."geniuxs_unsub.php?user=".$toid."&".$messagetype."=".$messageid."&email=".$accemail;
					$unsubscribelink = $unsubscribe;
					
					if (isset($mailformat) && $mailformat == "html") {
						$unsubscribe = "<a target=\"_blank\" href=\"".$unsubscribe."\">".$unsubscribe."</a>";
					}
					
					if (file_exists("geniuxs_unsub_text.php")) {
						include "geniuxs_unsub_text.php";
					}
					if (!isset($unsubtext) || strlen($unsubtext) < 1) {
						$unsubtext = "You are subscribed to receive updates with the e-mail address ".$accemail.". \nClick on the following link or paste it into your web browser to unsubscribe:";
					}
					
					$bodytext .= "\n\n\n".$unsubtext."\n\n".$unsubscribe;
					unset($unsubtext);

				} else {
					// Member account has been removed
					$subjecttext = "";
					$bodytext = "";
					AddQueueLog($prefix, "Deleting queue entry ".$queueid." because member account no longer exists");
				}

			} else {
				// Invalid Message
				$subjecttext = "";
				$bodytext = "";
				AddQueueLog($prefix, "Deleting queue entry ".$queueid." due to missing subject or message");
			}
			
		} else {
			// Could not find message
			lfmsql_query("DELETE FROM `geniuxs_queue` WHERE messageid=".$messageid) or die(lfmsql_error());
			$subjecttext = "";
			$bodytext = "";
			AddQueueLog($prefix, "Deleting queue entry ".$queueid." because message id ".$messageid." could not be found for queue type ".$queuetype);
		}
	
	} else {
		$getonemessage = lfmsql_query("SELECT subjecttext, bodytext FROM `geniuxs_queue` WHERE id=".$queueid." LIMIT 1") or die(lfmsql_error());
		$subjecttext = lfmsql_result($getonemessage, 0, "subjecttext");
		$bodytext = lfmsql_result($getonemessage, 0, "bodytext");
	}
	
	// Remove from the queue right before sending
	lfmsql_query("DELETE FROM `geniuxs_queue` WHERE id=".$queueid) or die(lfmsql_error());
	
	if ($subjecttext != "" && $bodytext != "") {
		// Send message
		
		if (stripos($bodytext, '<') !== FALSE && stripos($bodytext, '>') !== FALSE) {
			$mailformat = "html";
			if (stripos($bodytext, '<html') === FALSE) {
				$bodytext = nl2br($bodytext);
			}
		}
		
		if (!isset($mailformat) || strlen($mailformat) < 1) {
			$mailformat="plain";
		}
		
		if (strlen($bounceaddress) < 1) {
			$bounceaddress = $fromaddress;
		}
		
		$message = Swift_Message::newInstance();
		$message->setSubject($subjecttext);
		$message->setBody($bodytext);
		$message->setContentType('text/'.$mailformat);
		$message->setFrom(array($fromaddress => $fromname));
		$message->setTo($toaddress);
		$message->setReturnPath($bounceaddress);
		
		if (strlen($unsubscribelink) > 0) {
			$headers = $message->getHeaders();
			$headers->addTextHeader('List-Unsubscribe', '<'.$unsubscribelink.'>');
		}
		
		$result = $mailer->send($message);
		
	}
	
	$delivercount = $delivercount+1;
	
	if ($queuetype == 1 && is_numeric($messageid) && $messageid > 0) {
		lfmsql_query("UPDATE `geniuxs_schedsent` SET updatetime=".time().", numsends=numsends+1 WHERE messageid=".$messageid." LIMIT 1") or die(lfmsql_error());
	}
	
	lfmsql_query("UPDATE geniuxs_vals SET valvalue='".time()."' WHERE valtype='lastmail'") or die(lfmsql_error());
	
}

AddQueueLog($prefix, "Sent ".$delivercount." messages");
exit;

?>