<?php
// //////////////////////////////////////////////////////////////////////////
// LFM SQL API v1.2
// For TMS/LFM, LFMTE, and LFMVM scripts
// Written by Josh Abbott
// Copyright 2014-2016 AKH Media Group and Josh Abbott. All Rights Reserved.
// //////////////////////////////////////////////////////////////////////////

// These functions provide a layer between the script and PHP's database extensions,
// to provide a smoother transition from mysql to mysqli.  Developers should always
// use functions such as lfmsql_query() rather than mysql_query() or mysqli_query()
// so that alternative database extensions can be more easily supported in the future.

// Note that some parts of the script and apps still make direct calls to mysql
// functions at the current time, and some files will need to be updated to
// include this functions file (always use require_once to prevent multiple
// includes of a function file).


// This variable holds the PHP database extension that is being used.
// The script will default to mysqli and use mysql only if mysqli
// is not installed.
$GLOBALS['lfmsqlext'] = "mysqli";

// This variable sets whether the legacy mysql functions are being
// provided by the mysql compatibility feature of this script, or
// natively by the mysql extension.
$GLOBALS['mysqlcompatibility'] = false;

// This variable holds the database connection link identifier.
// This is optional with mysql but required with mysqli.
$GLOBALS['lfmsqllink'] = NULL;

// This variable holds the database connection link identifier
// for the mysql compatibility functions.  It works like the
// legacy mysql extension by holding the link of the last opened
// connection so it doesn't need to be passed to the functions
// every time.
$GLOBALS['mysqllink'] = NULL;


if (!function_exists('mysqli_connect')) {
	
	// If mysqli is not installed, the script will attempt to
	// provide some of the mysqli functions by running them
	// through the legacy mysql functions.
	
	if (!function_exists('mysql_connect')) {
		echo("Error: The mysql or mysqli PHP extension must be installed on this server.");
		exit;
	}
	
	$GLOBALS['lfmsqlext'] = "mysql";
	
	function mysqli_connect($sqlserver, $sqluser, $sqlpassword, $dbname='') {
		$connectlink = mysql_connect($sqlserver, $sqluser, $sqlpassword);
		if (strlen($dbname) > 0) {
			mysql_select_db($dbname, $connectlink);
		}
		return $connectlink;
	}
	
	function mysqli_select_db($sqllink, $sqldb) {
		return mysql_select_db($sqldb, $sqllink);
	}
	
	function mysqli_close($sqllink) {
		return mysql_close($sqllink);
	}
	
	function mysqli_error($sqllink) {
		return mysql_error($sqllink);
	}
	
	function mysqli_query($sqllink, $sqlquery) {
		return mysql_query($sqlquery, $sqllink);
	}
	
	function mysqli_data_seek($sqlresource, $sqlrow) {
		return mysql_data_seek($sqlresource, $sqlrow);
	}
	
	function mysqli_fetch_row($sqlresource) {
		return mysql_fetch_row($sqlresource);
	}
	
	function mysqli_num_rows($sqlresource) {
		return mysql_num_rows($sqlresource);
	}
	
	function mysqli_fetch_object($sqlresource) {
		return mysql_fetch_object($sqlresource);
	}
	
	function mysqli_fetch_array($sqlresource) {
		return mysql_fetch_array($sqlresource);
	}
	
	function mysqli_fetch_assoc($sqlresource) {
		return mysql_fetch_assoc($sqlresource);
	}
	
	function mysqli_insert_id($sqllink) {
		return mysql_insert_id($sqllink);
	}
	
	function mysqli_real_escape_string($sqllink, $strtxt) {
		return mysql_real_escape_string($strtxt, $sqllink);
	}
	
} elseif (!function_exists('mysql_connect')) {
	
	// If the mysql extension is not installed (such as in PHP 7)
	// the script will provide some of the most frequently used
	// mysql functions by running them through the equivalent
	// mysqli functions.
	
	$GLOBALS['mysqlcompatibility'] = true;
	
	function mysql_connect($sqlserver, $sqluser, $sqlpassword) {
		$link = mysqli_connect($sqlserver, $sqluser, $sqlpassword);
		$GLOBALS['mysqllink'] = $link;
		return $link;
	}
	
	function mysql_select_db($sqldb, $sqllink=0) {
		if ($sqllink == 0 && $GLOBALS['mysqllink'] === NULL) {
			// mysqli will not work without a link
			return false;
		} elseif ($sqllink == 0 && $GLOBALS['mysqllink'] !== NULL) {
			$result = mysqli_select_db($GLOBALS['mysqllink'], $sqldb);
		} else {
			$result = mysqli_select_db($sqllink, $sqldb);
		}
		return $result;
	}
	
	function mysql_close($sqllink=0) {
		if ($sqllink != 0) {
			mysqli_close($sqllink);
		} elseif ($GLOBALS['mysqllink'] !== NULL) {
			mysqli_close($GLOBALS['mysqllink']);
			$GLOBALS['mysqllink'] = NULL;
		}
		return true;
	}
	
	function mysql_error($sqllink=0) {
		if ($sqllink == 0 && $GLOBALS['mysqllink'] === NULL) {
			// mysqli will not work without a link
			return false;
		} elseif ($sqllink == 0 && $GLOBALS['mysqllink'] !== NULL) {
			$result = mysqli_error($GLOBALS['mysqllink']);
		} else {
			$result = mysqli_error($sqllink);
		}
		return $result;
	}
	
	function mysql_query($sqlquery, $sqllink=0) {
		if ($sqllink == 0 && $GLOBALS['mysqllink'] === NULL) {
			// mysqli will not work without a link
			return false;
		} elseif ($sqllink == 0 && $GLOBALS['mysqllink'] !== NULL) {
			$result = mysqli_query($GLOBALS['mysqllink'], $sqlquery);
		} else {
			$result = mysqli_query($sqllink, $sqlquery);
		}
		return $result;
	}
	
	function mysql_result($sqlresource, $sqlrow=0, $sqlfield=0) {
		$numrows = mysqli_num_rows($sqlresource);
		if ($numrows > 0 && $sqlrow <= ($numrows-1) && $sqlrow >= 0) {
			if (mysqli_data_seek($sqlresource, $sqlrow)) {
				if (is_numeric($sqlfield)) {
					// Return the entire row
					$result = mysqli_fetch_row($sqlresource);
				} else {
					// Return a specific field
					$result = mysqli_fetch_assoc($sqlresource);
				}
				if (isset($result[$sqlfield])) {
					return $result[$sqlfield];
				}
			}
		}
		// Invalid input or no results found
		return false;
	}
	
	function mysql_data_seek($sqlresource, $sqlrow) {
		$result = mysqli_data_seek($sqlresource, $sqlrow);
		return $result;
	}
	
	function mysql_fetch_row($sqlresource) {
		$result = mysqli_fetch_row($sqlresource);
		return $result;
	}
	
	function mysql_num_rows($sqlresource) {
		$result = mysqli_num_rows($sqlresource);
		return $result;
	}
	
	function mysql_fetch_object($sqlresource) {
		$result = mysqli_fetch_object($sqlresource);
		return $result;
	}
	
	function mysql_fetch_array($sqlresource) {
		$result = mysqli_fetch_array($sqlresource);
		return $result;
	}
	
	function mysql_fetch_assoc($sqlresource) {
		$result = mysqli_fetch_assoc($sqlresource);
		return $result;
	}
	
	function mysql_insert_id($sqllink=0) {
		if ($sqllink == 0 && $GLOBALS['mysqllink'] === NULL) {
			// mysqli will not work without a link
			return false;
		} elseif ($sqllink == 0 && $GLOBALS['mysqllink'] !== NULL) {
			$result = mysqli_insert_id($GLOBALS['mysqllink']);
		} else {
			$result = mysqli_insert_id($sqllink);
		}
		return $result;
	}
	
	function mysql_real_escape_string($strtxt, $sqllink=0) {
		if ($sqllink == 0 && $GLOBALS['mysqllink'] === NULL) {
			// mysqli will not work without a link
			return false;
		} elseif ($sqllink == 0 && $GLOBALS['mysqllink'] !== NULL) {
			$result = mysqli_real_escape_string($GLOBALS['mysqllink'], $strtxt);
		} else {
			$result = mysqli_real_escape_string($sqllink, $strtxt);
		}
		return $result;
	}
	
}


function lfmsql_get_ext_type() {
	// This function can be called if a developer needs to know
	// which database extension is currently being used.
	return $GLOBALS['lfmsqlext'];
}

function lfmsql_connect($sqlserver, $sqluser, $sqlpassword, $globallink=1) {
	if ($GLOBALS['lfmsqlext'] == "mysqli" && $GLOBALS['mysqlcompatibility'] == false) {
		// The mysql extension is installed so this opens a legacy
		// mysql connection for any old code that still uses it.
		mysql_connect($sqlserver, $sqluser, $sqlpassword);
	}
	// Open the main connection and return the link
	$link = mysqli_connect($sqlserver, $sqluser, $sqlpassword);
	if ($globallink == 1) {
		$GLOBALS['mysqllink'] = $GLOBALS['lfmsqllink'] = $link;
	}
	return $link;
}

function lfmsql_select_db($sqldb, $sqllink=0) {
	if ($GLOBALS['lfmsqlext'] == "mysqli" && $GLOBALS['mysqlcompatibility'] == false) {
		mysql_select_db($sqldb);
	}
	if ($sqllink == 0 && $GLOBALS['lfmsqllink'] === NULL) {
		// No link - attempt fallback to mysql
		$result = mysql_select_db($sqldb);
	} elseif ($sqllink == 0 && $GLOBALS['lfmsqllink'] !== NULL) {
		// Use the stored global link
		$result = mysqli_select_db($GLOBALS['lfmsqllink'], $sqldb);
	} else {
		// Use the link passed to the function
		$result = mysqli_select_db($sqllink, $sqldb);
	}
	return $result;
}

function lfmsql_close($sqllink=0) {
	if ($sqllink == 0 && $GLOBALS['lfmsqllink'] === NULL) {
		$result = mysql_close();
	} elseif ($sqllink == 0 && $GLOBALS['lfmsqllink'] !== NULL) {
		$result = mysqli_close($GLOBALS['lfmsqllink']);
		if ($GLOBALS['mysqllink'] == $GLOBALS['lfmsqllink']) {
			$GLOBALS['mysqllink'] = NULL;
		}
		$GLOBALS['lfmsqllink'] = NULL;
	} else {
		$result = mysqli_close($sqllink);
		if ($sqllink == $GLOBALS['lfmsqllink']) {
			if ($GLOBALS['mysqllink'] == $GLOBALS['lfmsqllink']) {
				$GLOBALS['mysqllink'] = NULL;
			}
			$GLOBALS['lfmsqllink'] = NULL;
		}
	}
	return $result;
}

function lfmsql_error($sqllink=0) {
	if ($sqllink == 0 && $GLOBALS['lfmsqllink'] === NULL) {
		$result = mysql_error();
	} elseif ($sqllink == 0 && $GLOBALS['lfmsqllink'] !== NULL) {
		$result = mysqli_error($GLOBALS['lfmsqllink']);
	} else {
		$result = mysqli_error($sqllink);
	}
	return $result;
}

function lfmsql_query($sqlquery, $sqllink=0) {
	if ($sqllink == 0 && $GLOBALS['lfmsqllink'] === NULL) {
		$result = mysql_query($sqlquery);
	} elseif ($sqllink == 0 && $GLOBALS['lfmsqllink'] !== NULL) {
		$result = mysqli_query($GLOBALS['lfmsqllink'], $sqlquery);
	} else {
		$result = mysqli_query($sqllink, $sqlquery);
	}
	return $result;
}

function lfmsql_result($sqlresource, $sqlrow=0, $sqlfield=0) {
	$numrows = mysqli_num_rows($sqlresource);
	if ($numrows > 0 && $sqlrow <= ($numrows-1) && $sqlrow >= 0) {
		if (mysqli_data_seek($sqlresource, $sqlrow)) {
			if (is_numeric($sqlfield)) {
				// Return the entire row
				$result = mysqli_fetch_row($sqlresource);
			} else {
				// Return a specific field
				$result = mysqli_fetch_assoc($sqlresource);
			}
			if (isset($result[$sqlfield])) {
				return $result[$sqlfield];
			}
		}
	}
	// Invalid input or no results found
	return false;
}

function lfmsql_num_rows($sqlresource) {
	$result = mysqli_num_rows($sqlresource);
	return $result;
}

function lfmsql_data_seek($sqlresource, $sqlrow) {
	$result = mysqli_data_seek($sqlresource, $sqlrow);
	return $result;
}

function lfmsql_fetch_row($sqlresource) {
	$result = mysqli_fetch_row($sqlresource);
	return $result;
}

function lfmsql_fetch_object($sqlresource) {
	$result = mysqli_fetch_object($sqlresource);
	return $result;
}

function lfmsql_fetch_array($sqlresource) {
	$result = mysqli_fetch_array($sqlresource);
	return $result;
}

function lfmsql_fetch_assoc($sqlresource) {
	$result = mysqli_fetch_assoc($sqlresource);
	return $result;
}

function lfmsql_insert_id($sqllink=0) {
	if ($sqllink == 0 && $GLOBALS['lfmsqllink'] === NULL) {
		$result = mysql_insert_id();
	} elseif ($sqllink == 0 && $GLOBALS['lfmsqllink'] !== NULL) {
		$result = mysqli_insert_id($GLOBALS['lfmsqllink']);
	} else {
		$result = mysqli_insert_id($sqllink);
	}
	return $result;
}

function lfmsql_real_escape_string($strtxt, $sqllink=0) {
	if ($sqllink == 0 && $GLOBALS['lfmsqllink'] === NULL) {
		$result = mysql_real_escape_string($strtxt);
	} elseif ($sqllink == 0 && $GLOBALS['lfmsqllink'] !== NULL) {
		$result = mysqli_real_escape_string($GLOBALS['lfmsqllink'], $strtxt);
	} else {
		$result = mysqli_real_escape_string($sqllink, $strtxt);
	}
	return $result;
}

?>