<?php

// SurfingGuard Checker v2.5
// 2011-2020 Josh Abbott, http://surfingguard.com

$rootdir=$_SERVER["DOCUMENT_ROOT"];

require_once "inc/filter.php";
require_once "inc/lfmsql.php";

include "inc/config.php";
lfmsql_connect($dbhost,$dbuser,$dbpass);
lfmsql_select_db($dbname) or die( "Unable to select database");

error_reporting(E_ALL ^ E_NOTICE ^ E_DEPRECATED);

set_time_limit(120);

ignore_user_abort(1);

require("class.socket.php");

class HCChecker {
  
var $pageErrors = array(
    "Host Not Found",
    "Page Not Found",
    "Too Many Redirects",
    "Has Hidden IFrame",
    "Has Encoded JavaScript",
    "Contains Frame Breaking Code",
    "Site In Banned List",
    "Invalid URL",
    "Site banned",
    "AntiVirus Server Down - Please try later",
    "Virus Detected",
    "Google - Suspected phishing page",
    "Google - Suspected malicious code",
    "Central Ban Server Down - Please try later",
    "Continuous Redirect Loop",
    "Malware Checker Down - Please try later",
    "Too Large - Over 200000 Characters Of Text"
    
);

var $pageWarnings = array(
    "Site Redirects",
    "Contains Frames",
    "Contains IFrames",
    "Contains Meta Refresh",
    "AntiVirus Server Down",
    "Central Ban Server Down",
    "Google Check Server Down"
);

var $errorsFound = array();
var $warningsFound = array();

//------------------------------------------------------------------------------------------------------------------------------------
// Program Start
//------------------------------------------------------------------------------------------------------------------------------------

var $redirectCount = 0;  
var $iFrameWidthMin = 20;    // The minimum acceptable width of an iframe
var $iFrameHeightMin = 20;   // The minimum acceptable height of an iframe
var $httpStatus = "";        // Holds the HTTP status of the page
var $host = "";
var $scheme = "";
var $checkstatus = "";
var $checkcount = 0;
var $banreasons = array();

var $actiontable = "";
var $iframecheck = "";
var $encodedscriptcheck = "";
var $centralbancheck = "";
var $viruscheck = "";
var $phishing = "";
var $malicious = "";

var $serverrequired = 0;

var $tename = "";
var $sgkey = "";

// Proxy server to use - If going through firewall
var $proxy = "15.191.1.50";
var $proxyPort = "8088";

function isMonitor($setmon) {
	if ($setmon == 1) {
		$this->actiontable = "sg_monitor_actions";
	}
	return;
}

function checkURL($url) {

  $this->serverrequired = lfmsql_result(lfmsql_query("Select value from sg_settings where name='serverrequired'"), 0);
  
  $this->tename = lfmsql_result(lfmsql_query("Select value from sg_settings where name='exchange'"), 0);
  $this->sgkey = lfmsql_result(lfmsql_query("Select value from sg_settings where name='key'"), 0);

  // Set the central ban types
  $this->banreasons[]=0;
  $checkbantypes = lfmsql_query("Select type from sg_ban_types");
  if (lfmsql_num_rows($checkbantypes) > 0) {
  	for ($i = 0; $i < lfmsql_num_rows($checkbantypes); $i++) {
  		$bantype = lfmsql_result($checkbantypes, $i, "type");
  		if (is_numeric($bantype)) {
	  		$this->banreasons[] = $bantype;
	  	}
  	}
  }

// Set the actions
if ($this->actiontable != "sg_monitor_actions") {
	$this->actiontable = "sg_checker_actions";
}
$this->iframecheck = lfmsql_result(lfmsql_query("Select action from ".$this->actiontable." where name='Hidden IFrame'"), 0);
$this->encodedscriptcheck = lfmsql_result(lfmsql_query("Select action from ".$this->actiontable." where name='Encoded JavaScript'"), 0);
$this->centralbancheck = lfmsql_result(lfmsql_query("Select action from ".$this->actiontable." where name='Site In Central Ban'"), 0);
$this->viruscheck = lfmsql_result(lfmsql_query("Select action from ".$this->actiontable." where name='Virus'"), 0);
$this->phishing = lfmsql_result(lfmsql_query("Select action from ".$this->actiontable." where name='Google - Suspected phishing page'"), 0);
$this->malicious = lfmsql_result(lfmsql_query("Select action from ".$this->actiontable." where name='Google - Suspected malicious code'"), 0);

  $this->checkstatus = "";
  
  if($this->valdateURL($url)) {
    if(!$this->bannedSites($url)) {
      if(!$this->approvedSites($url)) {
       $this->checkPage($url, true);
      }
    }
  }
  else
    $this->errorsFound[] = $this->pageErrors[7];

  
  $checkerResults = "<?xml version=\"1.0\" ?>\n";
  $checkerResults.= "<FOCUS_POINTS_PAGE_CHECK>";
  $checkerResults.= "<PAGE_CHECKED>".htmlentities($url)."</PAGE_CHECKED>";
  $checkerResults.= "<HTTP_STATUS>";
  $checkerResults.= "<HTTP_VERSION>".$this->httpStatus[1]."</HTTP_VERSION>";
  $checkerResults.= "<HTTP_STATUS>".$this->httpStatus[2]."</HTTP_STATUS>";
  $checkerResults.= "</HTTP_STATUS>";
  
  if(count($this->warningsFound) > 0) {
    $checkerResults.= "<PAGE_WARNINGS>";
    for($i=0; $i < count($this->warningsFound); $i++)
      $checkerResults.= "<WARNING_$i>".$this->warningsFound[$i]."</WARNING_$i>";
    $checkerResults.= "</PAGE_WARNINGS>";
  }
  
  if(count($this->errorsFound) > 0) {
    $checkerResults.= "<PAGE_ERRORS>";
    for($i=0; $i < count($this->errorsFound); $i++)
      $checkerResults.= "<ERROR_$i>".$this->errorsFound[$i]."</ERROR_$i>";
    $checkerResults.= "</PAGE_ERRORS>";
    $checkerResults.= "<PAGE_STATUS>FAILED</PAGE_STATUS>";
  }
  else 
    $checkerResults.= "<PAGE_STATUS>PASSED</PAGE_STATUS>";
  
  // Show Check Status
  $checkerResults.= "<CHECK_STATUS>";
  $checkerResults.= $this->checkstatus;
  $checkerResults.= "</CHECK_STATUS>";
  // End Show Check Status
  
  $checkerResults.= "</FOCUS_POINTS_PAGE_CHECK>";
  
  return $checkerResults;
  
}

function dumpPage($file, $data) {
  
     
  //check for existence of file
  if(file_exists($file) && is_writeable($file)) { $mode="a"; } else { $mode="w"; }
  
  //create file pointer
  $fp=fopen($file,$mode);
  
  fwrite($fp,"==================================================================================================\n"); 
  //write to file
  fwrite($fp,$data."\n"); 
  
  //close file pointer
  fclose($fp); 
  
  return true; 
  
}
//------------------------------------------------------------------------------------------------------------------------------------
// This is a function to call my socket class
//------------------------------------------------------------------------------------------------------------------------------------

function CallSocket($url,$timelimit=60) {

  $redirects = 0;
  $timeout = 30;
  $sock = new CSocket();
//  while(1) {

    set_time_limit($timelimit);
    $ar = parse_url($url);
    $query = '';
    if(isset($ar['host'])) $this->host  = $ar['host'];
    if(isset($ar['path'])) { $path  = $ar['path']; } else { $path='/'; }
    $path  = str_replace(' ','%20',$path);
    $path  = str_replace('\\','/',$path);
    if(isset($ar['query'])) { $query = $ar['query']; } else { $query =''; }
    //while(1) {
      $sock->timeouts($timeout,500,100,100);
      if($query == '') $hbuf = $this->GetSocketHeader($this->host, $path ,$path);      
      else $hbuf = $this->GetSocketHeader($this->host, $path . '?' . $query,$path);     
      for($retry=0;$retry<2;$retry++) {
        $sock->buffer = "";  
        $sock->proxyserver = $this->proxy;
        $sock->proxyport = $this->proxyPort;
        
        if (stripos($url, 'https://') !== false) {
		$connectport = 443;
		if (stripos($this->host, 'ssl://') === false) {
			$this->host = 'ssl://'.$this->host;
		}
	} else {
		$connectport = 80;
	}
        
        //$sock->proxyConnect($this->host,$connectport,1);
        $sock->Connect($this->host,$connectport,1);
        $sock->write($hbuf);
        $sock->receive_complete();
        if($sock->buffer <> "") break;
      }

//      $url = strstr($sock->buffer,"Location:");
// We will keep the HTTP header stuff please
//      if(strstr($sock->buffer,"Content-Type: text/html")) {
//        $sock->buffer = str_replace("\r\n","\n",$sock->buffer);
//        $sock->buffer = strstr($sock->buffer,"\n\n");
//      } else {
//        $sock->buffer = "";
//      }

//      if($url == "") {
        return $sock->buffer;
//      }
      // else
      // redirect happening
//      break;
//    }
//    $ar = explode("\n",$url);
//    $url = str_replace('Location:','',trim($ar[0]));
//    if(empty($url)) break;
//    if($redirects++ > 3) break;
//    $readcache = 0;

//  }
  //return $sock->buffer;
}

//------------------------------------------------------------------------------------------------------------------------------------
// This is called to get socket headers
//------------------------------------------------------------------------------------------------------------------------------------
function GetSocketHeader($domain,$path,$p) {
  $hbuf  = "GET $path HTTP/1.1\r\n";
  $hbuf .= "Host: $domain\r\n";
  $hbuf .= "Referer: http://".$_SERVER["SERVER_NAME"]."/\r\n";
  $hbuf .= "User-Agent: Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.88 Safari/537.36\r\n";
  $hbuf .= "Connection: close\r\n";
  $hbuf .= "Accept: */*\r\n" ;
  $hbuf .= "\r\n";
  return $hbuf;
}

function getHTTPStatus($pageHTML) {

  preg_match("/^(HTTP\/\d+\.\d+) (\d+) (.+)+/i",$pageHTML, $status);
  
  return $status;
}

function checkForFrames($pageHTML) {
  
  if(preg_match_all("/<frame (.+)>/i",$pageHTML, $frames)) {
    $this->warningsFound[] = $this->pageWarnings[1];  
    for($i=0; $i<count($frames[1]); $i++) {
      if(preg_match("/src=[\"']?([^\"' ><]+)/i",$frames[1][$i], $url))
        $this->checkPage($this->expandLocalUrl($url[1]));
    }
  }
}

function checkForIFrames($pageHTML) {
  
  if(preg_match_all("/<iframe (.+)>/i",$pageHTML, $iframes)) {
    $this->warningsFound[] = $this->pageWarnings[2];  
    for($i=0; $i<count($iframes[1]); $i++) {
    
     if ($this->iframecheck > 0) {
    
      // Get the Width
     if(preg_match("/ width=[\"']?([^\"' ><]+)/i",$iframes[1][$i], $width)) {
        if($width[1] < $this->iFrameWidthMin) {
          $this->errorsFound[] = $this->pageErrors[3];
          if (strpos($this->checkstatus, "Checking For Iframes") === false) {
	          $this->checkstatus .= "<STATUS_7>Checking For Iframes... SGFAILPNG</STATUS_7>";
	  }
          return;
        }
      }
      // Get the Height
      if(preg_match("/ height=[\"']?([^\"' ><]+)/i",$iframes[1][$i], $height)) {
        if($height[1] < $this->iFrameHeightMin) {
          $this->errorsFound[] = $this->pageErrors[3];
          if (strpos($this->checkstatus, "Checking For Iframes") === false) {
	          $this->checkstatus .= "<STATUS_7>Checking For Iframes... SGFAILPNG</STATUS_7>";
	  }
          return;
        }
      }
      
     }
      
      if(preg_match("/src=[\"']?([^\"' ><]+)/i",$iframes[1][$i], $url)) 
        $this->checkPage($this->expandLocalUrl($url[1]));
    }
  }
  if (strpos($this->checkstatus, "Checking For Iframes") === false) {
	  $this->checkstatus .= "<STATUS_7>Checking For Iframes... SGCHECKPNG</STATUS_7>";
  }
}

function checkForMetaRefresh($pageHTML) {

  if(preg_match_all("/<meta (.+)>/i",$pageHTML, $meta)) {
    for($i=0; $i<count($meta[1]); $i++) {
      // Check for a refresh
      if(preg_match("/http-equiv=[\"']?refresh[\"']?/i", $meta[1][$i]))
        $this->warningsFound[] = $this->pageWarnings[3];  
        if(preg_match("/url=[\"']?([^\"' ><]+)/i",$meta[1][$i], $url))
          $this->checkPage($url[1]);
    }
  }
}

function valdateURL($url) {
  
  // Remove any trailing GET variables
  $url = str_replace(stristr($url, '?'),'', $url);

  if( preg_match( '/^(http|https):\/\/[a-z0-9]+([\-\.]{1}[a-z0-9]+)*\.[a-z]{2,30}'
       .'((:[0-9]{1,5})?\/.*)?$/i' ,$url)) {
       $this->checkstatus .= "<STATUS_1>Validating URL... SGCHECKPNG</STATUS_1>";
	return true;
  }

  $this->checkstatus .= "<STATUS_1>Validating URL... SGFAILPNG</STATUS_1>";
  return false;
}

function bannedSites($url) {
  	  
  $bannedSites = lfmsql_query("SELECT domain from `banned_sites`");
  while ($sites = lfmsql_fetch_array($bannedSites)) {
	  if (stristr(trim($url), trim($sites['domain']))) {
	      $this->errorsFound[] = $this->pageErrors[6].": ".trim($sites['domain']);
	      $this->checkstatus .= "<STATUS_2>Banned URL... SGFAILPNG</STATUS_2>";
	      return true;
	  }
  }

  return false;
}

function approvedSites($url) {

  $approvedSites = lfmsql_query("SELECT domain from `approved_sites`");
  while ($sites = lfmsql_fetch_array($approvedSites)) {
	  if (stristr(trim($url), trim($sites['domain']))) {
      	      $this->checkstatus .= "<STATUS_3>Approved URL... SGCHECKPNG</STATUS_3>";
	      return true;
      	  }
  }

  return false;
}

function checkPage($url, $topLevel = false) {
  
//echo "<br>Checking Page -> $url ";

  if($this->checkcount > 12) {
	  $this->errorsFound[] = $this->pageErrors[14];
  } else {
  
  $this->checkcount = $this->checkcount + 1;

  if(!$this->bannedSites($url)) {
  
  $ar = parse_url($url);
  
  if(isset($ar['host'])) $this->host  = $ar['host'];
  if(isset($ar['scheme'])) $this->scheme  = $ar['scheme'];
      
  if(($ip =gethostbyname($this->host)) == $this->host) {
    if($topLevel)
      $this->errorsFound[] =  $this->pageErrors[0];
      if (strpos($this->checkstatus, "Connecting To Site") === false) {
	      $this->checkstatus .= "<STATUS_4>Connecting To Site... SGFAILPNG</STATUS_4>";
      }
    else  
      $this->warningsFound[] =  $this->pageErrors[0];
      if (strpos($this->checkstatus, "Connecting To Site") === false) {
	      $this->checkstatus .= "<STATUS_4>Connecting To Site... SGFAILPNG</STATUS_4>";
      }
  }
  else {
  
    // WORK OUT TIME TO LOAD BY HOW LONG IT TAKES TO GET DATA  
    $page = $this->CallSocket($url);
    //$this->dumpPage("file", $page);
    
    // If no data is returned, the connection must have failed
    if (strlen($page) < 1) {
	    if($topLevel)
	      $this->errorsFound[] =  $this->pageErrors[0];
	      if (strpos($this->checkstatus, "Connecting To Site") === false) {
		      $this->checkstatus .= "<STATUS_4>Connecting To Site... SGFAILPNG</STATUS_4>";
	      }
	    else  
	      $this->warningsFound[] =  $this->pageErrors[0];
	      if (strpos($this->checkstatus, "Connecting To Site") === false) {
		      $this->checkstatus .= "<STATUS_4>Connecting To Site... SGFAILPNG</STATUS_4>";
	      }
    } else {
    
    if (strpos($this->checkstatus, "Connecting To Site") === false) {
	    $this->checkstatus .= "<STATUS_4>Connecting To Site... SGCHECKPNG</STATUS_4>";
    }
    
    // Get the HTTP Status
    $this->httpStatus = $this->getHTTPStatus($page);
    
    // Client Error, no need to carry on
    if($this->httpStatus[2][0] == "4") {
      if($topLevel)
        $this->warningsFound[] =  $this->pageErrors[1];
        if (strpos($this->checkstatus, "Checking Page") === false) {
	        $this->checkstatus .= "<STATUS_5>Checking Page... SGFAILPNG</STATUS_5>";
	}
      else
        $this->warningsFound[] =  $this->pageErrors[1];
        if (strpos($this->checkstatus, "Checking Page") === false) {
	        $this->checkstatus .= "<STATUS_5>Checking Page... SGFAILPNG</STATUS_5>";
	}
      return;
    }
    
    // Server Error, no need to carry on
    if($this->httpStatus[2][0] == "5") {
      if($topLevel)
        $this->warningsFound[] =  $this->pageErrors[1];
        if (strpos($this->checkstatus, "Checking Page") === false) {
	        $this->checkstatus .= "<STATUS_5>Checking Page... SGFAILPNG</STATUS_5>";
	}
      else
        $this->warningsFound[] =  $this->pageErrors[1];
        if (strpos($this->checkstatus, "Checking Page") === false) {
	        $this->checkstatus .= "<STATUS_5>Checking Page... SGFAILPNG</STATUS_5>";
	}
      return;
    }
    
    $pagelen = strlen($page);
    if ($pagelen > 500000) {
    	$this->errorsFound[] =  $this->pageErrors[16];
    	$this->checkstatus .= "<STATUS_5>Checking Page... SGFAILPNG</STATUS_5>";
    	return;
    }
    
    if (strpos($this->checkstatus, "Checking Page") === false) {
	    $this->checkstatus .= "<STATUS_5>Checking Page... SGCHECKPNG</STATUS_5>";
    }
    
    
    // No need to continue if the site already failed
    if(count($this->errorsFound) == 0) {
    
    // Run Server Side Tests
    if($this->runServerTests($url, $this->serverrequired)) {

    // Check for redirects
    if($this->httpStatus[2][0] == "3") {
      $this->warningsFound[] = $this->pageWarnings[0];
      $url = stristr($page ,"Location:");
      if($this->redirectCount++ > 8) {
        $this->errorsFound[] =  $this->pageErrors[2];
        if (strpos($this->checkstatus, "Looking For Redirects") === false) {
	        $this->checkstatus .= "<STATUS_6>Looking For Redirects... SGFAILPNG</STATUS_6>";
	}
        return;
      }
      $ar = explode("\n",$url);
      $url = str_ireplace('Location:','',trim($ar[0]));
      $this->checkPage($this->expandLocalUrl($url));
    }
    
    // Check for META refreshes
    $this->checkForMetaRefresh($page);
    
    // Check for Frames
    $this->checkForFrames($page);
      
    // Check for IFrames including sizes etc
    $this->checkForIFrames($page);
    
    $this->checkHTML($page);
    
    }
    
    }
    
  }
  
  }
  
  }
  
  }
}

function checkHTML($pageHTML) {
  
//echo "<br>checkHTML-><pre>".$pageHTML."</pre>";
  // Check page for "unescaped" characters
  if(preg_match_all("/unescape[ ]*[(]?[\"']?([^\"']+)/i",$pageHTML, $unescape)) {
    for($i=0; $i<count($unescape[1]); $i++) {
      $this->checkHTML(urldecode($unescape[1][$i]));
    }
  }

  // Check Javascript - get all the script tags 
  if(preg_match_all("/(<script.*?>)+?(.*)<\/script/is",$pageHTML, $scripts)) {
    //print_r($scripts[1]);
    for($i=0; $i<count($scripts[1]); $i++) {
     if ($this->encodedscriptcheck > 0) {
    
      // Check for encoded Javascript      
      if(preg_match("/JScript\.Encode/i", $scripts[1][$i])) {
        $this->errorsFound[] = $this->pageErrors[4];
        if (strpos($this->checkstatus, "Scanning JavaScript") === false) {
	        $this->checkstatus .= "<STATUS_8>Scanning JavaScript... SGFAILPNG</STATUS_8>";
	}
        return;
      }
            
     }
    }
    
    // Get all external JavaSript links and check them
    // Need to deal with Src= for Javascript loaded off site    
    
    // Now check the actual JavaScript code
    for($i=0; $i<count($scripts[2]); $i++) {
       $this->checkJavaScript($scripts[2][$i]);  
    }        
  }
  if (strpos($this->checkstatus, "Scanning JavaScript") === false) {
  	$this->checkstatus .= "<STATUS_8>Scanning JavaScript... SGCHECKPNG</STATUS_8>";
  }
}

function checkJavaScript($pageScript) {

//echo "Checking Scripts-><pre>$pageScript</pre>";       
    
    // Check javascript for    
    // 1: Framebreaking code
    if(preg_match("/p[\n\r'\"+ ]*?a[\n\r'\"+ ]*?r[\n\r'\"+ ]*?e[\n\r'\"+ ]*?n[\n\r'\"+ ]*?t[\n\r'\"+ ]*?[ ]*?\.[ ]*?[\n\r'\"+ ]*?l[\n\r'\"+ ]*?o[\n\r'\"+ ]*?c[\n\r'\"+ ]*?a[\n\r'\"+ ]*?t[\n\r'\"+ ]*?i[\n\r'\"+ ]*?o[\n\r'\"+ ]*?n[\n\r'\"+ ]*?[ ]*?\.[ ]*?[\n\r'\"+ ]*?h[\n\r'\"+ ]*?r[\n\r'\"+ ]*?e[\n\r'\"+ ]*?f[\n\r'\"+ ]*?[ ]*[\n\r'\"+ ]*?=[\n\r\n\r'\"+ ]*?[ ]*[\n\r\n\r'\"+ ]*?s[\n\r'\"+ ]*?e[\n\r'\"+ ]*?l[\n\r'\"+ ]*?f[\n\r'\"+ ]*?[ ]*?\.[ ]*?[\n\r'\"+ ]*?d[\n\r'\"+ ]*?o[\n\r'\"+ ]*?c[\n\r'\"+ ]*?u[\n\r'\"+ ]*?m[\n\r'\"+ ]*?e[\n\r'\"+ ]*?n[\n\r'\"+ ]*?t[\n\r'\"+ ]*?[ ]*?\.[ ]*?[\n\r'\"+ ]*?l[\n\r'\"+ ]*?o[\n\r'\"+ ]*?c[\n\r'\"+ ]*?a[\n\r'\"+ ]*?t[\n\r'\"+ ]*?i[\n\r'\"+ ]*?o[\n\r'\"+ ]*?n[\n\r'\"+ ]*?/is", $pageScript)) {
      $this->errorsFound[] = $this->pageErrors[5];
      if (strpos($this->checkstatus, "Scanning JavaScript") === false) {
	      $this->checkstatus .= "<STATUS_8>Scanning JavaScript... SGFAILPNG</STATUS_8>";
      }
      }
}

function centralBanExclude($url) {

  $excluded = lfmsql_query("SELECT domain from `sg_ban_exclude`");
  while ($sites = lfmsql_fetch_array($excluded)) {
	  if (stristr(trim($url), trim($sites['domain'])))
      return true;
  }
	
  return false;
}

function expandLocalUrl($url) {

  $url = trim($url);
  $rURL = parse_url($url);
  if(!isset($rURL['host'])) {
    // Strip any leading '/' from the URL
    if(preg_match("/\/(.+)/i",$url, $sURL))
      $url = $sURL[1];

    if(isset($this->scheme))        
      $url = $this->scheme."://".$this->host."/".$url;
     else
      $url = "http://".$this->host."/".$url;
  }      

  return $url;
}   

function getURLfromQueryString($queryString) {
 
 $url = urldecode($queryString);
 
 $url = substr($url, 2);
 
 return $url;  
} 


function runServerTests($url, $serverrequired=0) {

	if ($this->phishing == 0 && $this->malicious == 0) {
		$googlecheck = 0;
	} else {
		$googlecheck = 1;
	}

	if($this->centralBanExclude($url)) {
		$bancheck = 0;
	} else {
		if ($this->centralbancheck > 0) {
			$bancheck = 1;
		} else {
			$bancheck = 0;
		}
	}
	
	if ($this->viruscheck > 0) {
		$checkforvirus = 1;
	} else {
		$checkforvirus = 0;
	}
	
	reset($this->banreasons);
	foreach($this->banreasons AS $bankey=>$banval) {
		$bantypes .= $banval."A";
	}
	$bantypes = substr($bantypes, 0, strlen($bantypes)-1);

	$runtestsurl = "https://sgchecker.com/runchecks.php?bancheck=".$bancheck."&googlecheck=".$googlecheck."&viruscheck=".$checkforvirus."&exchange=".$this->tename."&key=".$this->sgkey."&bantypes=".$bantypes."&url=".urlencode($url);
	
	// Make sure the server is online
	$serveronline = 1;
	
	$urlinfo = parse_url($runtestsurl);
	$hostname = $urlinfo['host'];
	if ($hostname == "" || gethostbyname($hostname) == $hostname) {
		$serveronline = 0;
	} else {
	$port = 443;
	$timeout = 7;
	$headers = "GET ".$urlinfo['path'].'?'.$urlinfo['query']." HTTP/1.1\r\n";
	$headers .= "Host: ".$hostname."\r\n";
	$headers .= "Referer: http://".$_SERVER["SERVER_NAME"]."\r\n";
	$headers .= "User-Agent: Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.88 Safari/537.36\r\n";
	$headers .= "Connection: close\r\n";
	$headers .= "Accept: */*\r\n";
	$headers .= "\r\n";
	$testreturn = "";
	$attempts = 0;
	while((strpos($testreturn, '200 OK') === false) && (strpos($testreturn, '302 Found') === false) && ($attempts<2)) {
		$connection = fsockopen('ssl://'.$hostname, $port, $errno, $errstr, $timeout);
		if ($connection) {
			stream_set_timeout($connection, $timeout);
			fwrite($connection,$headers);
			//Read the reply
			$testreturn = "";
			$readruns = 0;
			while (!feof($connection) && !$checktimeout['timed_out'] && $readruns < 500) {
				$testreturn .= fread($connection, 10000);
				$checktimeout = stream_get_meta_data($connection);
				$readruns++;
			}
			fclose($connection);
		}
		$attempts = $attempts+1;
	}
	if((strpos($testreturn, '200 OK') === false) && (strpos($testreturn, '302 Found') === false)) {
	$serveronline = 0;
	}
	}
	
	if ($serveronline == 1) {
	
		$bannedchecked = 1;
		$googlechecked = 1;
		$viruschecked = 1;
		
		if(strpos($testreturn, 'banned') === false) {
			$bannedchecked = 0;
		}
		
		if(strpos($testreturn, 'Google') === false) {
			$googlechecked = 0;
		}
		
		if(strpos($testreturn, 'irus') === false) {
			$viruschecked = 0;
		}
		
	} else {
		$bannedchecked = 0;
		$googlechecked = 0;
		$viruschecked = 0;
	}
	
	if ($bancheck == 1) {
		if ($bannedchecked == 1) {
			if (strpos($testreturn, "Site banned") !== false) {
	  			$colonpos = strpos($testreturn, "Site banned");
				$this->errorsFound[] = substr($testreturn, $colonpos);
				$this->checkstatus .= "<STATUS_2>Banned Site... SGFAILPNG</STATUS_2>";
				return false;
			}
		} else {
			if ($serverrequired == 0) {
				// Return Warning
				$this->warningsFound[] = $this->pageWarnings[5];
				
			} else {
				// Return Error
				$this->errorsFound[] = $this->pageErrors[13];
				if (strpos($this->checkstatus, "Checking Banned Sites") === false) {
					$this->checkstatus .= "<STATUS_2>Checking Banned Sites... SGFAILPNG</STATUS_2>";
				}			
				return false;
			}
		}
	}
	
	if ($googlecheck == 1) {
		if ($googlechecked == 1) {

			if ($this->phishing > 0 && strpos($testreturn, "Blacklist") !== false) {
		  		$this->errorsFound[] = $this->pageErrors[11];
				if (strpos($this->checkstatus, "Checking For Phishing/Malware") === false) {
					$this->checkstatus .= "<STATUS_9>Checking For Phishing/Malware... SGFAILPNG</STATUS_9>";
				}
				return false;
				
			} elseif ($this->malicious > 0 && strpos($testreturn, "Malware") !== false) {
		  		$this->errorsFound[] = $this->pageErrors[12];
				if (strpos($this->checkstatus, "Checking For Phishing/Malware") === false) {
					$this->checkstatus .= "<STATUS_9>Checking For Phishing/Malware... SGFAILPNG</STATUS_9>";
				}
				return false;
				
			} else {
				if (strpos($this->checkstatus, "Checking For Phishing/Malware") === false) {
					$this->checkstatus .= "<STATUS_9>Checking For Phishing/Malware... SGCHECKPNG</STATUS_9>";
				}
			}

		} else {
			if ($serverrequired == 0) {
				// Return Warning
				$this->warningsFound[] = $this->pageWarnings[6];
				
			} else {
				// Return Error
				$this->errorsFound[] = $this->pageErrors[15];
				if (strpos($this->checkstatus, "Checking For Phishing/Malware") === false) {
					$this->checkstatus .= "<STATUS_9>Checking For Phishing/Malware... SGFAILPNG</STATUS_9>";
				}			
				return false;
			}
		}
	}
	
	if ($checkforvirus == 1) {
		if ($viruschecked == 1) {

			if (strpos($testreturn, "Virus Found") !== false) {
		  		$colonpos = strpos($testreturn, "Virus Found:");
			    	$virusname = substr($testreturn, $colonpos);
				$this->errorsFound[] = $virusname;
				if (strpos($this->checkstatus, "Scanning For Viruses") === false) {
					$this->checkstatus .= "<STATUS_10>Scanning For Viruses... SGFAILPNG</STATUS_10>";
				}
				return false;
			} else {
				if (strpos($this->checkstatus, "Scanning For Viruses") === false) {
					$this->checkstatus .= "<STATUS_10>Scanning For Viruses... SGCHECKPNG</STATUS_10>";
				}
			}

		} else {
			if ($serverrequired == 0) {
				// Return Warning
				$this->warningsFound[] = $this->pageWarnings[4];
				
			} else {
				// Return Error
				$this->errorsFound[] = $this->pageErrors[9];
				if (strpos($this->checkstatus, "Scanning For Viruses") === false) {
					$this->checkstatus .= "<STATUS_10>Scanning For Viruses... SGFAILPNG</STATUS_10>";
				}			
				return false;
			}
		}
	}

  return true;
}

}
?>