// SECTION 2A SCRIPTS


// Get references to the phone number input fields
const primPhoneInput = document.getElementById('prim-phone');
const homePhoneInput = document.getElementById('home-phone');
const workPhoneInput = document.getElementById('work-phone');
const altPhoneInput = document.getElementById('alt-cont-num');
const kinPhoneInput = document.getElementById('kin-num');

// Add input event listeners to format phone numbers as XXX-XXX-XXXX
primPhoneInput.addEventListener('input', formatPhoneNumber);
homePhoneInput.addEventListener('input', formatPhoneNumber);
workPhoneInput.addEventListener('input', formatPhoneNumber);
altPhoneInput.addEventListener('input', formatPhoneNumber);
kinPhoneInput.addEventListener('input', formatPhoneNumber);

// Function to format phone numbers as XXX-XXX-XXXX
function formatPhoneNumber() {
    this.value = this.value.replace(/\D/g, ''); // Remove non-numeric characters
    const match = this.value.match(/^(\d{0,3})(\d{0,3})(\d{0,4})$/);
    if (match) {
        this.value = match[1] + (match[1] && match[2] ? '-' : '') + match[2] + (match[2] && match[3] ? '-' : '') + match[3];
    }
    }
    
        // Get references to the form and add a submit event listener
const form = document.querySelector('form');
form.addEventListener('submit', function (e) {
    if (!atLeastOnePhoneNumberProvided()) {
        e.preventDefault(); // Prevent form submission if no phone number is provided
        alert('At least one phone number is required.');
    }
});

// Function to check if at least one phone number is provided --- NEED TO FIX
function atLeastOnePhoneNumberProvided() {
    return primPhoneInput.value.trim() !== '' || homePhoneInput.value.trim() !== '' || workPhoneInput.value.trim() !== '';
}

// Get references to the checkbox and email input fields
const noEmailCheckbox = document.getElementById('no-email');
const primEmailInput = document.getElementById('prim-email');
const workEmailInput = document.getElementById('work-email');

// Get references to the labels
const primEmailLabel = document.querySelector('label[for="prim-email"]');
const workEmailLabel = document.querySelector('label[for="work-email"]');

// Add an event listener to the checkbox
noEmailCheckbox.addEventListener('change', function() {
    // If the checkbox is checked, disable the email input fields and update labels
    if (this.checked) {
        primEmailInput.disabled = true;
        workEmailInput.disabled = true;
        primEmailLabel.textContent = 'Primary Email';
        workEmailLabel.textContent = 'Work Email';
    } else {
        // If the checkbox is unchecked, enable the email input fields and update labels
        primEmailInput.disabled = false;
        workEmailInput.disabled = false;
        primEmailLabel.textContent = 'Primary Email *';
        workEmailLabel.textContent = 'Work Email *';
    }

    // Update required attribute of email input fields based on checkbox state
    updateEmailFieldRequired();
});

// Function to update the required attribute of email input fields based on checkbox state
function updateEmailFieldRequired() {
    if (noEmailCheckbox.checked) {
        primEmailInput.removeAttribute('required');
        workEmailInput.removeAttribute('required');
    } else {
        primEmailInput.setAttribute('required', 'required');
        workEmailInput.setAttribute('required', 'required');
    }
}


// SECTION 2B SCRIPTS


    // Get references to the checkbox and emergency input fields
const noEmergCheckbox = document.getElementById('no-emerg');
const emergNameInput = document.getElementById('alt-cont-name');
const emergeRelateInput = document.getElementById('alt-cont-relate');
const emergeCellInput = document.getElementById('alt-cont-num');
const emergeEmailInput = document.getElementById('alt-cont-email');

// Get references to the labels
const emergeNameLabel = document.querySelector('label[for="alt-cont-name"]');
const emergeRelateLabel = document.querySelector('label[for="alt-cont-relate"]');
const emergeCellLabel = document.querySelector('label[for="alt-cont-num"]');
const emergeEmailLabel = document.querySelector('label[for="alt-cont-email"]');

// Add an event listener to the checkbox
noEmergCheckbox.addEventListener('change', function() {
    // If the checkbox is checked, disable the email input fields and update labels
    if (this.checked) {
        emergNameInput.disabled = true;
        emergeRelateInput.disabled = true;
        emergeCellInput.disabled = true;
        emergeEmailInput.disabled = true;
        emergeNameLabel.textContent = 'Emergency Contact Name';
        emergeRelateLabel.textContent = 'Relationship';
        emergeCellLabel.textContent = 'Phone Number';
        emergeEmailLabel.textContent = 'Email Address';
    } else {
        // If the checkbox is unchecked, enable the email input fields and update labels
        emergNameInput.disabled = false;
        emergeRelateInput.disabled = false;
        emergeCellInput.disabled = false;
        emergeEmailInput.disabled = false;
        emergeNameLabel.textContent = 'Emergency Contact Name';
        emergeRelateLabel.textContent = 'Relationship';
        emergeCellLabel.textContent = 'Phone Number';
        emergeEmailLabel.textContent = 'Email Address';
    }
    });

document.addEventListener('DOMContentLoaded', function() {
const sameKinCheckbox = document.getElementById('same-kin');
const emergNameInput = document.getElementById('alt-cont-name');
const emergRelateInput = document.getElementById('alt-cont-relate');
const emergCellInput = document.getElementById('alt-cont-num');
const emergEmailInput = document.getElementById('alt-cont-email');

const kinNameInput = document.getElementById('kin-name');
const kinRelateInput = document.getElementById('kin-relate');
const kinCellInput = document.getElementById('kin-num');
const kinEmailInput = document.getElementById('kin-email');

    // Get references to the labels
const kinNameLabel = document.querySelector('label[for="kin-name"]');
const kinRelateLabel = document.querySelector('label[for="kin-relate"]');
const kinCellLabel = document.querySelector('label[for="kin-num"]');
const kinEmailLabel = document.querySelector('label[for="kin-email"]');


// Add an event listener to the "Check if Same as Emergency Contact" checkbox
sameKinCheckbox.addEventListener('change', function() {
    if (this.checked) {
        // Copy values from Emergency Contact fields to Next of Kin fields
        kinNameInput.value = emergNameInput.value;
        kinRelateInput.value = emergRelateInput.value;
        kinCellInput.value = emergCellInput.value;
        kinEmailInput.value = emergEmailInput.value;
        
        // Disables fields
        kinNameInput.disabled = true;
        kinRelateInput.disabled = true;
        kinCellInput.disabled = true;
        kinEmailInput.disabled = true;
        kinNameLabel.textContent = 'Next of Kin Name';
        kinRelateLabel.textContent = 'Relationship';
        kinCellLabel.textContent = 'Phone Number';
        kinEmailLabel.textContent = 'Email Address';
    } else {
        // Clear the Next of Kin fields if the checkbox is unchecked
        kinNameInput.value = '';
        kinRelateInput.value = '';
        kinCellInput.value = '';
        kinEmailInput.value = '';
        
        // Enables fields
        kinNameInput.disabled = false;
        kinRelateInput.disabled = false;
        kinCellInput.disabled = false;
        kinEmailInput.disabled = false;
        kinNameLabel.textContent = 'Next of Kin Name';
        kinRelateLabel.textContent = 'Relationship';
        kinCellLabel.textContent = 'Phone Number';
        kinEmailLabel.textContent = 'Email Address';
    }
});
});


const commAltCheckbox = document.getElementById('comm-alt-checkbox');
const commAltHiddenInput = document.querySelector('input[name="comm-alt"]');

commAltCheckbox.addEventListener('change', function() {
    if (this.checked) {
        commAltHiddenInput.value = 'YES'; // Set the value to "YES" when the checkbox is checked
    } else {
        commAltHiddenInput.value = 'NO'; // Set the value back to "NO" when the checkbox is unchecked
    }
});