let currentSection = 1;

const sectionOrder = [
        "section1",
        "section2",
        "section3",
        "section4",
        "section5"
    ];

function prevSection() {
    if (currentSection > 1) {
            currentSection--;
            updateSectionVisibility();
            updateButtonVisibility();
            window.scrollTo(0, 0);
        }
}

function nextSection() {
    if (currentSection === 1 && validateSection("section1")) {
        currentSection++;
        updateSectionVisibility();
        updateButtonVisibility();
        window.scrollTo(0, 0);
    } else if (currentSection === 2 && validateSection("section2")) {
        currentSection++;
        updateSectionVisibility();
        updateButtonVisibility();
        window.scrollTo(0, 0);
    } else if (currentSection === 3 && validateSection("section3")) {
        currentSection++;
        updateSectionVisibility();
        updateButtonVisibility();
        window.scrollTo(0, 0);    
    } else if (currentSection === 4 && validateSection("section4")) {
        currentSection++;
        updateSectionVisibility();
        updateButtonVisibility();
        window.scrollTo(0, 0);
    } else if (currentSection === 5 && validateSection("section5")) {
        currentSection++;
        updateSectionVisibility();
        updateButtonVisibility();
        window.scrollTo(0, 0);  
  }
}

function checkRequiredFieldsInSection(section) {
    const requiredFields = section.querySelectorAll('[required]');
    let allFieldsFilled = true;
    for (const field of requiredFields) {
        if (!field.value.trim()) {
            allFieldsFilled = false;
            break;
        }
    }

    // If all fields are filled, clear the notification
    if (allFieldsFilled) {
        hideNotification();
    }
}

// The validateSection function checks if all required fields in a section are filled out
function validateSection(sectionId) {
    const section = document.getElementById(sectionId);
    const requiredFields = section.querySelectorAll('[required]');
    let firstInvalidField = null;

    for (const field of requiredFields) {
        const label = document.querySelector(`label[for="${field.id}"]`);
        if (!field.value.trim()) {
            if (label) label.classList.add('highlight');
            if (!firstInvalidField) firstInvalidField = field;
        } else {
            if (label) label.classList.remove('highlight');
        }
    }

    if (firstInvalidField) {
        firstInvalidField.scrollIntoView({ behavior: 'smooth', block: 'center' });
        firstInvalidField.focus();
        showNotification("Please fill out all required fields, in red, and Press NEXT again.");
        return false;
    } else {
        hideNotification(); // This is called if all required fields are filled out
    }

     // Specific validation for section2 (phone numbers)
     if (sectionId === "section2") {
        const primPhone = document.getElementById("prim-phone").value;
        const homePhone = document.getElementById("home-phone").value;
        const workPhone = document.getElementById("work-phone").value;

        if (primPhone.trim() === "" && homePhone.trim() === "" && workPhone.trim() === "") {
            showNotification("Please enter at least one phone number.");
            return false;
        }
    }

    hideNotification();
    return true;
}

function showNotification(message) {
    const notification = document.getElementById("notification");
    if (notification) {
        notification.textContent = message;
        notification.style.display = "block";
    }
}

function hideNotification() {
    const notification = document.getElementById('notification');
    if (notification) {
        notification.textContent = ''; // Clear the message
        notification.style.display = 'none'; // Hide the notification element
    }
}

function updateSectionVisibility() {
    // Array of section IDs in the order you want to navigate

    // Loop through the sections and update their display based on currentSection
    for (let i = 0; i < sectionOrder.length; i++) {
        const sectionId = sectionOrder[i];
        const section = document.getElementById(sectionId);

        // Determine whether to display or hide the section
        const shouldDisplay = i === currentSection - 1;
        section.style.display = shouldDisplay ? "block" : "none";
    }
}
    
    function updateButtonVisibility() {
    const prevButton = document.getElementById("prevButton");
    const nextButton = document.getElementById("nextButton");
    const submitButton = document.getElementById("submitButton");

    // Show or hide the Previous button
    if (currentSection === 1) {
        prevButton.style.display = "none";
    } else {
        prevButton.style.display = "inline-block";
    }

    // Show or hide the Next button
    if (currentSection === sectionOrder.length) {
        nextButton.style.display = "none";
    } else {
        nextButton.style.display = "inline-block";
    }

    // Show or hide the Submit button
    if (currentSection === sectionOrder.length) {
        submitButton.style.display = "inline-block";
    } else {
        submitButton.style.display = "none";
    }
}